@@ -67,6 +67,7 @@
 #include "src/common/macros.h"
 #include "src/common/net.h"
 #include "src/common/plugstack.h"
+#include "src/common/read_config.h"
 #include "src/common/slurm_auth.h"
 #include "src/common/slurm_cred.h"
 #include "src/common/slurm_mpi.h"
@@ -99,7 +100,6 @@ static void _print_launch_msg(launch_tasks_request_msg_t *msg,
 /**********************************************************************
  * Message handler declarations
  **********************************************************************/
-static uid_t  slurm_uid;
 static bool   force_terminated_job = false;
 static int    task_exit_signal = 0;
 
@@ -169,7 +169,12 @@ static void _rebuild_mpi_layout(slurm_step_ctx_t *ctx,
 {
 	slurm_step_layout_t *new_step_layout, *orig_step_layout;
 
-	ctx->launch_state->mpi_info->het_job_id = params->het_job_id;
+	if (params->het_job_offset == NO_VAL)
+		return;
+
+	if (params->het_job_id && (params->het_job_id != NO_VAL))
+		ctx->launch_state->mpi_info->het_job_id = params->het_job_id;
+
 	ctx->launch_state->mpi_info->het_job_task_offset =
 		params->het_job_task_offset;
 	new_step_layout = xmalloc(sizeof(slurm_step_layout_t));
@@ -232,8 +237,7 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 		return SLURM_ERROR;
 	}
 
-	if (params->het_job_id && (params->het_job_id != NO_VAL))
-		_rebuild_mpi_layout(ctx, params);
+	_rebuild_mpi_layout(ctx, params);
 
 	mpi_env = xmalloc(sizeof(char *));  /* Needed for setenvf used by MPI */
 	if ((ctx->launch_state->mpi_state =
@@ -250,7 +254,8 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 		return rc;
 
 	/* Start tasks on compute nodes */
-	launch.job_id = ctx->step_req->job_id;
+	memcpy(&launch.step_id, &ctx->step_req->step_id,
+	       sizeof(launch.step_id));
 	launch.uid = ctx->step_req->user_id;
 	launch.gid = params->gid;
 	launch.argc = params->argc;
@@ -258,7 +263,6 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 	launch.spank_job_env = params->spank_job_env;
 	launch.spank_job_env_size = params->spank_job_env_size;
 	launch.cred = ctx->step_resp->cred;
-	launch.job_step_id = ctx->step_resp->job_step_id;
 	launch.het_job_node_offset = params->het_job_node_offset;
 	launch.het_job_step_cnt = params->het_job_step_cnt;
 	launch.het_job_id = params->het_job_id;
@@ -314,8 +318,10 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 	if (params->multi_prog)
 		launch.flags	|= LAUNCH_MULTI_PROG;
 	launch.cpus_per_task	= params->cpus_per_task;
+	launch.threads_per_core	= params->threads_per_core;
 	launch.ntasks_per_board = params->ntasks_per_board;
 	launch.ntasks_per_core  = params->ntasks_per_core;
+	launch.ntasks_per_tres  = params->ntasks_per_tres;
 	launch.ntasks_per_socket= params->ntasks_per_socket;
 
 	if (params->no_alloc)
@@ -383,7 +389,7 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 		 * if io_timeout seconds pass without stdio traffic to/from
 		 * the node.
 		 */
-		ctx->launch_state->io_timeout = slurm_get_msg_timeout();
+		ctx->launch_state->io_timeout = slurm_conf.msg_timeout;
 	} else { /* user_managed_io is true */
 		/* initialize user_managed_io_t */
 		ctx->launch_state->io.user =
@@ -448,7 +454,8 @@ extern int slurm_step_launch_add(slurm_step_ctx_t *ctx,
 	memset(&launch, 0, sizeof(launch));
 
 	/* Start tasks on compute nodes */
-	launch.job_id = ctx->step_req->job_id;
+	memcpy(&launch.step_id, &ctx->step_req->step_id,
+	       sizeof(launch.step_id));
 	launch.uid = ctx->step_req->user_id;
 	launch.gid = params->gid;
 	launch.argc = params->argc;
@@ -456,7 +463,6 @@ extern int slurm_step_launch_add(slurm_step_ctx_t *ctx,
 	launch.spank_job_env = params->spank_job_env;
 	launch.spank_job_env_size = params->spank_job_env_size;
 	launch.cred = ctx->step_resp->cred;
-	launch.job_step_id = ctx->step_resp->job_step_id;
 	launch.het_job_step_cnt = params->het_job_step_cnt;
 	launch.het_job_id = params->het_job_id;
 	launch.het_job_nnodes = params->het_job_nnodes;
@@ -576,7 +582,7 @@ extern int slurm_step_launch_add(slurm_step_ctx_t *ctx,
 		 * if io_timeout seconds pass without stdio traffic to/from
 		 * the node.
 		 */
-		ctx->launch_state->io_timeout = slurm_get_msg_timeout();
+		ctx->launch_state->io_timeout = slurm_conf.msg_timeout;
 	} else { /* user_managed_io is true */
 		xrealloc(ctx->launch_state->io.user->sockets,
 			 sizeof(int) * ctx->step_req->num_tasks);
@@ -707,19 +713,17 @@ void slurm_step_launch_wait_finish(slurm_step_ctx_t *ctx)
 				sls->abort_action_taken = true;
 			}
 			if (!time_set) {
-				uint16_t kill_wait;
 				/* Only set the time once, because we only want
 				 * to wait STEP_ABORT_TIME, no matter how many
 				 * times the condition variable is signaled.
 				 */
-				kill_wait = slurm_get_kill_wait();
 				ts.tv_sec = time(NULL) + STEP_ABORT_TIME
-					+ kill_wait;
+					+ slurm_conf.kill_wait;
 				time_set = true;
 				/* FIXME - should this be a callback? */
 				info("Job step aborted: Waiting up to "
 				     "%d seconds for job step to finish.",
-				     kill_wait + STEP_ABORT_TIME);
+				     slurm_conf.kill_wait + STEP_ABORT_TIME);
 			}
 
 			errnum = pthread_cond_timedwait(&sls->cond,
@@ -757,8 +761,7 @@ void slurm_step_launch_wait_finish(slurm_step_ctx_t *ctx)
 		info("Job step aborted");	/* no need to wait */
 
 	if (!force_terminated_job && task_exit_signal)
-		info("Force Terminated job step %u.%u",
-		     ctx->job_id, ctx->step_resp->job_step_id);
+		info("Force Terminated %ps", &ctx->step_req->step_id);
 
 	/*
 	 * task_exit_signal != 0 when srun receives a message that a task
@@ -852,8 +855,7 @@ extern void slurm_step_launch_fwd_signal(slurm_step_ctx_t *ctx, int signo)
 
 	/* common to all tasks */
 	memset(&msg, 0, sizeof(msg));
-	msg.job_id      = ctx->job_id;
-	msg.job_step_id = ctx->step_resp->job_step_id;
+	memcpy(&msg.step_id, &ctx->step_req->step_id, sizeof(msg.step_id));
 	msg.signal      = (uint16_t) signo;
 
 	slurm_mutex_lock(&sls->lock);
@@ -891,8 +893,8 @@ extern void slurm_step_launch_fwd_signal(slurm_step_ctx_t *ctx, int signo)
 	slurm_mutex_unlock(&sls->lock);
 
 	if (!hostlist_count(hl)) {
-		verbose("no active tasks in step %u.%u to send signal %d",
-		        ctx->job_id, ctx->step_resp->job_step_id, signo);
+		verbose("no active tasks in %ps to send signal %d",
+		        &ctx->step_req->step_id, signo);
 		hostlist_destroy(hl);
 		return;
 	}
@@ -900,14 +902,15 @@ extern void slurm_step_launch_fwd_signal(slurm_step_ctx_t *ctx, int signo)
 	hostlist_destroy(hl);
 
 RESEND:	slurm_msg_t_init(&req);
+	slurm_msg_set_r_uid(&req, SLURM_AUTH_UID_ANY);
 	req.msg_type = REQUEST_SIGNAL_TASKS;
 	req.data     = &msg;
 
 	if (ctx->step_resp->use_protocol_ver)
 		req.protocol_version = ctx->step_resp->use_protocol_ver;
 
-	debug2("sending signal %d to step %u.%u on hosts %s",
-	       signo, ctx->job_id, ctx->step_resp->job_step_id, name);
+	debug2("sending signal %d to %ps on hosts %s",
+	       signo, &ctx->step_req->step_id, name);
 
 	if (!(ret_list = slurm_send_recv_msgs(name, &req, 0))) {
 		error("fwd_signal: slurm_send_recv_msgs really failed badly");
@@ -927,9 +930,10 @@ RESEND:	slurm_msg_t_init(&req);
 		    (rc != ESLURMD_JOB_NOTRUNNING) && (rc != ESRCH) &&
 		    (rc != EAGAIN) &&
 		    (rc != ESLURM_TRANSITION_STATE_NO_UPDATE)) {
-			error("Failure sending signal %d to step %u.%u on node %s: %s",
-			      signo, ctx->job_id, ctx->step_resp->job_step_id,
-			      ret_data_info->node_name, slurm_strerror(rc));
+			error("Failure sending signal %d to %ps on node %s: %s",
+			      signo, &ctx->step_req->step_id,
+			      ret_data_info->node_name,
+			      slurm_strerror(rc));
 		}
 		if ((rc == EAGAIN) || (rc == ESLURM_TRANSITION_STATE_NO_UPDATE))
 			retry = true;
@@ -973,10 +977,10 @@ struct step_launch_state *step_launch_state_create(slurm_step_ctx_t *ctx)
 	sls->abort = false;
 	sls->abort_action_taken = false;
 	/* NOTE: No malloc() of sls->mpi_info required */
-	sls->mpi_info->jobid = ctx->step_req->job_id;
+	memcpy(&sls->mpi_info->step_id, &ctx->step_req->step_id,
+	       sizeof(sls->mpi_info->step_id));
 	sls->mpi_info->het_job_id = NO_VAL;
 	sls->mpi_info->het_job_task_offset = NO_VAL;
-	sls->mpi_info->stepid = ctx->step_resp->job_step_id;
 	sls->mpi_info->step_layout = layout;
 	sls->mpi_state = NULL;
 	slurm_mutex_init(&sls->lock);
@@ -1142,13 +1146,10 @@ static int _msg_thr_create(struct step_launch_state *sls, int num_nodes)
 	eio_obj_t *obj;
 	int i, rc = SLURM_SUCCESS;
 	uint16_t *ports;
-	uint16_t eio_timeout;
 
 	debug("Entering _msg_thr_create()");
-	slurm_uid = (uid_t) slurm_get_slurm_user_id();
 
-	eio_timeout = slurm_get_srun_eio_timeout();
-	sls->msg_handle = eio_handle_create(eio_timeout);
+	sls->msg_handle = eio_handle_create(slurm_conf.eio_timeout);
 	sls->num_resp_port = _estimate_nports(num_nodes, 48);
 	sls->resp_port = xmalloc(sizeof(uint16_t) * sls->num_resp_port);
 
@@ -1156,7 +1157,7 @@ static int _msg_thr_create(struct step_launch_state *sls, int num_nodes)
 	 * parallel jobs using PMI sometimes result in slow message
 	 * responses and timeouts. Raise the default timeout for srun. */
 	if (!message_socket_ops.timeout)
-		message_socket_ops.timeout = slurm_get_msg_timeout() * 8000;
+		message_socket_ops.timeout = slurm_conf.msg_timeout * 8000;
 
 	ports = slurm_get_srun_port_range();
 	for (i = 0; i < sls->num_resp_port; i++) {
@@ -1229,10 +1230,10 @@ _exit_handler(struct step_launch_state *sls, slurm_msg_t *exit_msg)
 	void (*task_finish)(task_exit_msg_t *);
 	int i;
 
-	if ((msg->job_id != sls->mpi_info->jobid) ||
-	    (msg->step_id != sls->mpi_info->stepid)) {
-		debug("Received MESSAGE_TASK_EXIT from wrong job: %u.%u",
-		      msg->job_id, msg->step_id);
+	if ((msg->step_id.job_id != sls->mpi_info->step_id.job_id) ||
+	    (msg->step_id.step_id != sls->mpi_info->step_id.step_id)) {
+		debug("Received MESSAGE_TASK_EXIT from wrong job: %ps",
+		      &msg->step_id);
 		return;
 	}
 
@@ -1270,8 +1271,7 @@ _job_complete_handler(struct step_launch_state *sls, slurm_msg_t *complete_msg)
 		verbose("Complete job %u received",
 			step_msg->job_id);
 	} else {
-		verbose("Complete job step %u.%u received",
-			step_msg->job_id, step_msg->step_id);
+		verbose("Complete %ps received", &step_msg->step_id);
 	}
 
 	if (sls->callback.step_complete)
@@ -1390,9 +1390,8 @@ _step_missing_handler(struct step_launch_state *sls, slurm_msg_t *missing_msg)
 	int   num_tasks;
 	bool  active;
 
-	debug("Step %u.%u missing from node(s) %s",
-	      step_missing->job_id, step_missing->step_id,
-	      step_missing->nodelist);
+	debug("Step %ps missing from node(s) %s",
+	      &step_missing->step_id, step_missing->nodelist);
 
 	/* Ignore this message in the unusual "user_managed_io" case.  No way
 	   to confirm a bad connection, since a test message goes straight to
@@ -1508,8 +1507,8 @@ static void
 _step_step_signal(struct step_launch_state *sls, slurm_msg_t *signal_msg)
 {
 	job_step_kill_msg_t *step_signal = signal_msg->data;
-	debug2("Signal %u requested for step %u.%u", step_signal->signal,
-	       step_signal->job_id, step_signal->job_step_id);
+	debug2("Signal %u requested for step %ps", step_signal->signal,
+	       &step_signal->step_id);
 	if (sls->callback.step_signal)
 		(sls->callback.step_signal)(step_signal->signal);
 
@@ -1569,7 +1568,8 @@ _handle_msg(void *arg, slurm_msg_t *msg)
 
 	req_uid = g_slurm_auth_get_uid(msg->auth_cred);
 
-	if ((req_uid != slurm_uid) && (req_uid != 0) && (req_uid != uid)) {
+	if ((req_uid != slurm_conf.slurm_user_id) && (req_uid != 0) &&
+	    (req_uid != uid)) {
 		error ("Security violation, slurm message from uid %u",
 		       (unsigned int) req_uid);
  		return;
@@ -1667,9 +1667,7 @@ static int _fail_step_tasks(slurm_step_ctx_t *ctx, char *node, int ret_code)
 	slurm_mutex_unlock(&sls->lock);
 
 	memset(&msg, 0, sizeof(msg));
-	msg.job_id = ctx->job_id;
-	msg.job_step_id = ctx->step_resp->job_step_id;
-
+	memcpy(&msg.step_id, &ctx->step_req->step_id, sizeof(msg.step_id));
 	msg.range_first = msg.range_last = nodeid;
 	msg.step_rc = ret_code;
 
@@ -1718,11 +1716,12 @@ static int _launch_tasks(slurm_step_ctx_t *ctx,
 	 * running Prolog
 	 */
 	if (timeout <= 0) {
-		timeout = (slurm_get_msg_timeout() +
-			   slurm_get_batch_start_timeout()) * 1000;
+		timeout = (slurm_conf.msg_timeout +
+		           slurm_conf.batch_start_timeout) * 1000;
 	}
 
 	slurm_msg_t_init(&msg);
+	slurm_msg_set_r_uid(&msg, SLURM_AUTH_UID_ANY);
 	msg.msg_type = REQUEST_LAUNCH_TASKS;
 	msg.data = launch_msg;
 
@@ -1757,10 +1756,9 @@ static int _launch_tasks(slurm_step_ctx_t *ctx,
 
 			errno = tot_rc;
 			tot_rc = SLURM_ERROR;
-			error("Task launch for %u.%u failed on "
-			      "node %s: %m",
-			      ctx->job_id, ctx->step_resp->job_step_id,
-			      ret_data->node_name);
+			error("Task launch for %ps failed on node %s: %m",
+			      &ctx->step_req->step_id, ret_data->node_name);
+
 		} else {
 #if 0 /* only for debugging, might want to make this a callback */
 			errno = ret_data->err;
@@ -1804,9 +1802,8 @@ static void _print_launch_msg(launch_tasks_request_msg_t *msg,
 	task_list = hostlist_ranged_string_xmalloc(hl);
 	hostlist_destroy(hl);
 
-	info("launching %u.%u on host %s, %u tasks: %s",
-	     msg->job_id, msg->job_step_id, hostname,
-	     msg->tasks_to_launch[nodeid], task_list);
+	info("launching %ps on host %s, %u tasks: %s",
+	     &msg->step_id, hostname, msg->tasks_to_launch[nodeid], task_list);
 	xfree(task_list);
 
 	debug3("uid:%ld gid:%ld cwd:%s %d", (long) msg->uid,
@@ -1829,13 +1826,12 @@ _exec_prog(slurm_msg_t *msg)
 		error("%s: called with no command to execute", __func__);
 		return;
 	} else if (exec_msg->argc > 2) {
-		verbose("Exec '%s %s' for %u.%u",
+		verbose("Exec '%s %s' for %ps",
 			exec_msg->argv[0], exec_msg->argv[1],
-			exec_msg->job_id, exec_msg->step_id);
+			&exec_msg->step_id);
 	} else {
-		verbose("Exec '%s' for %u.%u",
-			exec_msg->argv[0],
-			exec_msg->job_id, exec_msg->step_id);
+		verbose("Exec '%s' for %ps",
+			exec_msg->argv[0], &exec_msg->step_id);
 	}
 
 	if (pipe(pfd) == -1) {
@@ -1858,6 +1854,7 @@ _exec_prog(slurm_msg_t *msg)
 		close(pfd[1]);
 		execvp(exec_msg->argv[0], exec_msg->argv);
 		error("execvp(%s): %m", exec_msg->argv[0]);
+		_exit(127);
 	} else if (child < 0) {
 		snprintf(buf, sizeof(buf), "fork: %s", strerror(errno));
 		error("%s", buf);
