@@ -40,6 +40,7 @@
 #include "src/common/gres.h"
 #include "src/common/log.h"
 #include "src/common/list.h"
+#include "src/common/read_config.h"
 #include "ctype.h"
 
 #include <nvml.h>
@@ -96,9 +97,8 @@ static bitstr_t	*saved_gpus = NULL;
  * (major.minor.micro combined into a single number).
  */
 const char	*plugin_name		= "GPU NVML plugin";
-const char	*plugin_type		= "gpu/nvml";
+const char	plugin_type[]		= "gpu/nvml";
 const uint32_t	plugin_version		= SLURM_VERSION_NUMBER;
-static log_level_t log_lvl              = LOG_LEVEL_DEBUG5;
 
 
 /*
@@ -535,15 +535,15 @@ static void _get_nearest_freq(unsigned int *freq, unsigned int freqs_size,
 	unsigned int i;
 
 	if (!freq || !(*freq)) {
-		log_var(log_lvl, "%s: No frequency supplied", __func__);
+		log_flag(GRES, "%s: No frequency supplied", __func__);
 		return;
 	}
 	if (!freqs || !(*freqs)) {
-		log_var(log_lvl, "%s: No frequency list supplied", __func__);
+		log_flag(GRES, "%s: No frequency list supplied", __func__);
 		return;
 	}
 	if (freqs_size <= 0) {
-		log_var(log_lvl, "%s: Frequency list is empty", __func__);
+		log_flag(GRES, "%s: Frequency list is empty", __func__);
 		return;
 	}
 
@@ -579,32 +579,32 @@ static void _get_nearest_freq(unsigned int *freq, unsigned int freqs_size,
 
 	/* check if freq is out of bounds of freqs */
 	if (*freq > freqs[0]) {
-		log_var(log_lvl, "Rounding requested frequency %u MHz down to "
-			"%u MHz (highest available)", *freq, freqs[0]);
+		log_flag(GRES, "Rounding requested frequency %u MHz down to %u MHz (highest available)",
+		         *freq, freqs[0]);
 		*freq = freqs[0];
 		return;
 	} else if (*freq < freqs[freqs_size - 1]) {
-		log_var(log_lvl, "Rounding requested frequency %u MHz up to %u "
-			"MHz (lowest available)", *freq, freqs[freqs_size - 1]);
+		log_flag(GRES, "Rounding requested frequency %u MHz up to %u MHz (lowest available)",
+		         *freq, freqs[freqs_size - 1]);
 		*freq = freqs[freqs_size - 1];
 		return;
 	}
 
 	/* check for frequency, and round up if no exact match */
 	for (i = 0; i < freqs_size - 1;) {
-		if (*freq == freqs[i])
+		if (*freq == freqs[i]) {
 			// No change necessary
 			debug2("No change necessary. Freq: %u MHz", *freq);
 			return;
+		}
 		i++;
 		/*
 		 * Step down to next element to round up.
 		 * Safe to advance due to bounds checks above here
 		 */
 		if (*freq > freqs[i]) {
-			log_var(log_lvl, "Rounding requested frequency %u MHz "
-				"up to %u MHz (next available)", *freq,
-				freqs[i - 1]);
+			log_flag(GRES, "Rounding requested frequency %u MHz up to %u MHz (next available)",
+			         *freq, freqs[i - 1]);
 			*freq = freqs[i - 1];
 			return;
 		}
@@ -814,17 +814,16 @@ static void _reset_freq(bitstr_t *gpus)
 		// TODO: Check to make sure that the frequency reset
 
 		if (freq_reset) {
-			log_var(log_lvl, "Successfully reset GPU[%d]", i);
+			log_flag(GRES, "Successfully reset GPU[%d]", i);
 			count_set++;
 		} else {
-			log_var(log_lvl, "Failed to reset GPU[%d]", i);
+			log_flag(GRES, "Failed to reset GPU[%d]", i);
 		}
 	}
 
 	if (count_set != count) {
-		log_var(log_lvl,
-			"%s: Could not reset frequencies for all GPUs. "
-			"Set %d/%d total GPUs", __func__, count_set, count);
+		log_flag(GRES, "%s: Could not reset frequencies for all GPUs. Set %d/%d total GPUs",
+		         __func__, count_set, count);
 		fprintf(stderr, "Could not reset frequencies for all GPUs. "
 			"Set %d/%d total GPUs\n", count_set, count);
 	}
@@ -852,7 +851,6 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	bool task_cgroup = false;
 	bool constrained_devices = false;
 	bool cgroups_active = false;
-	char *task_plugin_type = NULL;
 
 	/*
 	 * Parse frequency information
@@ -882,10 +880,8 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	slurm_mutex_unlock(&xcgroup_config_read_mutex);
 
 	// Check if task/cgroup plugin is loaded
-	task_plugin_type = slurm_get_task_plugin();
-	if (strstr(task_plugin_type, "cgroup"))
+	if (xstrstr(slurm_conf.task_plugin, "cgroup"))
 		task_cgroup = true;
-	xfree(task_plugin_type);
 
 	// If both of these are true, then GPUs will be constrained
 	if (constrained_devices && task_cgroup) {
@@ -938,10 +934,10 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 		}
 
 		if (freq_set) {
-			log_var(log_lvl, "Successfully set GPU[%d] %s", i, tmp);
+			log_flag(GRES, "Successfully set GPU[%d] %s", i, tmp);
 			count_set++;
 		} else {
-			log_var(log_lvl, "Failed to set GPU[%d] %s", i, tmp);
+			log_flag(GRES, "Failed to set GPU[%d] %s", i, tmp);
 		}
 
 		if (verbose_flag && !freq_logged) {
@@ -952,9 +948,8 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	}
 
 	if (count_set != count) {
-		log_var(log_lvl,
-			"%s: Could not set frequencies for all GPUs. "
-			"Set %d/%d total GPUs", __func__, count_set, count);
+		log_flag(GRES, "%s: Could not set frequencies for all GPUs. Set %d/%d total GPUs",
+		         __func__, count_set, count);
 		fprintf(stderr, "Could not set frequencies for all GPUs. "
 			"Set %d/%d total GPUs\n", count_set, count);
 	}
@@ -1274,7 +1269,7 @@ static List _get_system_gpu_list_nvml(node_config_load_t *node_config)
 		if (!_nvml_get_handle(i, &device)) {
 			error("Creating null GRES GPU record");
 			add_gres_to_list(gres_list_system, "gpu", 1,
-					 node_config->cpu_cnt, NULL,
+					 node_config->cpu_cnt, NULL, NULL,
 					 NULL, NULL, NULL);
 			continue;
 		}
@@ -1294,12 +1289,12 @@ static List _get_system_gpu_list_nvml(node_config_load_t *node_config)
 
 		// Convert from bitstr_t to cpu range str
 		cpu_aff_mac_range = bit_fmt_full(cpu_aff_mac_bitstr);
-		bit_free(cpu_aff_mac_bitstr);
 
 		// Convert cpu range str from machine to abstract(slurm) format
 		if (node_config->xcpuinfo_mac_to_abs(cpu_aff_mac_range,
 						     &cpu_aff_abs_range)) {
 			error("    Conversion from machine to abstract failed");
+			FREE_NULL_BITMAP(cpu_aff_mac_bitstr);
 			xfree(cpu_aff_mac_range);
 			continue;
 		}
@@ -1319,15 +1314,19 @@ static List _get_system_gpu_list_nvml(node_config_load_t *node_config)
 		if (minor_number != i)
 			debug("Note: GPU index %u is different from minor "
 			      "number %u", i, minor_number);
-		debug2("    CPU Affinity Range: %s", cpu_aff_mac_range);
-		debug2("    CPU Affinity Range Abstract: %s",cpu_aff_abs_range);
+		debug2("    CPU Affinity Range - Machine: %s",
+		       cpu_aff_mac_range);
+		debug2("    Core Affinity Range - Abstract: %s",
+		       cpu_aff_abs_range);
 		// Print out possible memory frequencies for this device
 		_nvml_print_freqs(device, LOG_LEVEL_DEBUG2);
 
 		add_gres_to_list(gres_list_system, "gpu", 1,
 				 node_config->cpu_cnt, cpu_aff_abs_range,
-				 device_file, device_name, nvlinks);
+				 cpu_aff_mac_bitstr, device_file, device_name,
+				 nvlinks);
 
+		FREE_NULL_BITMAP(cpu_aff_mac_bitstr);
 		xfree(cpu_aff_mac_range);
 		xfree(cpu_aff_abs_range);
 		xfree(nvlinks);
@@ -1350,9 +1349,6 @@ extern int init(void)
 {
 	debug("%s: %s loaded", __func__, plugin_name);
 
-	if (slurm_get_debug_flags() & DEBUG_FLAG_GRES)
-		log_lvl = LOG_LEVEL_INFO;
-
 	return SLURM_SUCCESS;
 }
 
@@ -1365,11 +1361,6 @@ extern int fini(void)
 
 extern int gpu_p_reconfig(void)
 {
-	if (slurm_get_debug_flags() & DEBUG_FLAG_GRES)
-		log_lvl = LOG_LEVEL_INFO;
-	else
-		log_lvl = LOG_LEVEL_DEBUG5;
-
 	return SLURM_SUCCESS;
 }
 
