@@ -41,7 +41,8 @@
 #include "src/common/slurm_xlator.h"
 #include "src/common/gres.h"
 #include "src/common/log.h"
-#include <rocm_smi.h>
+#include "src/common/read_config.h"
+#include <rocm_smi/rocm_smi.h>
 
 /*
  * #defines needed to test rsmi.
@@ -110,17 +111,13 @@ typedef struct rsmiPciInfo_st {
  * (major.minor.micro combined into a single number).
  */
 const char	*plugin_name		= "GPU RSMI plugin";
-const char	*plugin_type		= "gpu/rsmi";
+const char	plugin_type[]		= "gpu/rsmi";
 const uint32_t	plugin_version		= SLURM_VERSION_NUMBER;
-static log_level_t log_lvl              = LOG_LEVEL_DEBUG5;
 
 extern int init(void)
 {
 	debug("%s: %s loaded", __func__, plugin_name);
 
-	if (slurm_get_debug_flags() & DEBUG_FLAG_GRES)
-		log_lvl = LOG_LEVEL_INFO;
-
 	return SLURM_SUCCESS;
 }
 
@@ -428,15 +425,15 @@ static void _get_nearest_freq(unsigned int *freq, unsigned int freqs_size,
 	unsigned int i;
 
 	if (!freq || !(*freq)) {
-		log_var(log_lvl, "%s: No frequency supplied", __func__);
+		log_flag(GRES, "%s: No frequency supplied", __func__);
 		return;
 	}
 	if (!freqs || !(*freqs)) {
-		log_var(log_lvl, "%s: No frequency list supplied", __func__);
+		log_flag(GRES, "%s: No frequency list supplied", __func__);
 		return;
 	}
 	if (freqs_size <= 0) {
-		log_var(log_lvl, "%s: Frequency list is empty", __func__);
+		log_flag(GRES, "%s: Frequency list is empty", __func__);
 		return;
 	}
 
@@ -472,31 +469,32 @@ static void _get_nearest_freq(unsigned int *freq, unsigned int freqs_size,
 
 	/* check if freq is out of bounds of freqs */
 	if (*freq > freqs[0]) {
-		log_var(log_lvl, "Rounding frequency %u MHz down to %u MHz",
-			*freq, freqs[0]);
+		log_flag(GRES, "Rounding frequency %u MHz down to %u MHz",
+		         *freq, freqs[0]);
 		*freq = freqs[0];
 		return;
 	} else if (*freq < freqs[freqs_size - 1]) {
-		log_var(log_lvl, "Rounding frequency %u MHz up to %u MHz",
-			*freq, freqs[freqs_size - 1]);
+		log_flag(GRES, "Rounding frequency %u MHz up to %u MHz",
+		         *freq, freqs[freqs_size - 1]);
 		*freq = freqs[freqs_size - 1];
 		return;
 	}
 
 	/* check for frequency, and round up if no exact match */
 	for (i = 0; i < freqs_size - 1;) {
-		if (*freq == freqs[i])
+		if (*freq == freqs[i]) {
 			// No change necessary
 			debug2("No change necessary. Freq: %u MHz", *freq);
-		return;
+			return;
+		}
 		i++;
 		/*
 		 * Step down to next element to round up.
 		 * Safe to advance due to bounds checks above here
 		 */
 		if (*freq > freqs[i]) {
-			log_var(log_lvl, "Rounding frequency %u MHz up to %u MHz",
-				*freq, freqs[i - 1]);
+			log_flag(GRES, "Rounding frequency %u MHz up to %u MHz",
+			         *freq, freqs[i - 1]);
 			*freq = freqs[i - 1];
 			return;
 		}
@@ -759,17 +757,16 @@ static void _reset_freq(bitstr_t *gpus)
 		// TODO: Check to make sure that the frequency reset
 
 		if (freq_reset) {
-			log_var(log_lvl, "Successfully reset GPU[%d]", i);
+			log_flag(GRES, "Successfully reset GPU[%d]", i);
 			count_set++;
 		} else {
-			log_var(log_lvl, "Failed to reset GPU[%d]", i);
+			log_flag(GRES, "Failed to reset GPU[%d]", i);
 		}
 	}
 
 	if (count_set != count) {
-		log_var(log_lvl,
-			"%s: Could not reset frequencies for all GPUs %d/%d total GPUs",
-			__func__, count_set, count);
+		log_flag(GRES, "%s: Could not reset frequencies for all GPUs %d/%d total GPUs",
+		         __func__, count_set, count);
 		fprintf(stderr, "Could not reset frequencies for all GPUs %d/%d total GPUs\n",
 			count_set, count);
 	}
@@ -798,7 +795,6 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	bool task_cgroup = false;
 	bool constrained_devices = false;
 	bool cgroups_active = false;
-	char *task_plugin_type = NULL;
 
 	// Parse frequency information
 	debug2("_parse_gpu_freq(%s)", gpu_freq);
@@ -826,10 +822,8 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	slurm_mutex_unlock(&xcgroup_config_read_mutex);
 
 	// Check if task/cgroup plugin is loaded
-	task_plugin_type = slurm_get_task_plugin();
-	if (strstr(task_plugin_type, "cgroup"))
+	if (xstrstr(slurm_conf.task_plugin, "cgroup"))
 		task_cgroup = true;
-	xfree(task_plugin_type);
 
 	// If both of these are true, then GPUs will be constrained
 	if (constrained_devices && task_cgroup) {
@@ -878,10 +872,10 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 		}
 
 		if (freq_set) {
-			log_var(log_lvl, "Successfully set GPU[%d] %s", i, tmp);
+			log_flag(GRES, "Successfully set GPU[%d] %s", i, tmp);
 			count_set++;
 		} else {
-			log_var(log_lvl, "Failed to set GPU[%d] %s", i, tmp);
+			log_flag(GRES, "Failed to set GPU[%d] %s", i, tmp);
 		}
 
 		if (verbose_flag && !freq_logged) {
@@ -892,9 +886,8 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	}
 
 	if (count_set != count) {
-		log_var(log_lvl,
-			"%s: Could not set frequencies for all GPUs %d/%d total GPUs",
-			__func__, count_set, count);
+		log_flag(GRES, "%s: Could not set frequencies for all GPUs %d/%d total GPUs",
+		         __func__, count_set, count);
 		fprintf(stderr, "Could not set frequencies for all GPUs %d/%d total GPUs\n",
 			count_set, count);
 	}
@@ -1107,7 +1100,7 @@ static List _get_system_gpu_list_rsmi(node_config_load_t *node_config)
 		_rsmi_print_freqs(i, LOG_LEVEL_DEBUG2);
 
 		add_gres_to_list(gres_list_system, "gpu", 1,
-				 node_config->cpu_cnt, NULL,
+				 node_config->cpu_cnt, NULL, NULL,
 				 device_file, device_brand, NULL);
 
 		xfree(device_file);
@@ -1121,15 +1114,9 @@ static List _get_system_gpu_list_rsmi(node_config_load_t *node_config)
 
 extern int gpu_p_reconfig(void)
 {
-	if (slurm_get_debug_flags() & DEBUG_FLAG_GRES)
-		log_lvl = LOG_LEVEL_INFO;
-	else
-		log_lvl = LOG_LEVEL_DEBUG5;
-
 	return SLURM_SUCCESS;
 }
 
-
 extern List gpu_p_get_system_gpu_list(node_config_load_t *node_config)
 {
 	List gres_list_system = _get_system_gpu_list_rsmi(node_config);
