@@ -62,6 +62,10 @@
 
 #include "task_cgroup.h"
 
+typedef struct {
+	stepd_step_rec_t *job;
+} task_memory_create_callback_t;
+
 static char user_cgroup_path[PATH_MAX];
 static char job_cgroup_path[PATH_MAX];
 static char jobstep_cgroup_path[PATH_MAX];
@@ -77,7 +81,7 @@ static void _calc_device_major(char *dev_path[PATH_MAX],
 			       char *dev_major[PATH_MAX],
 			       int lines);
 
-static int _read_allowed_devices_file(char *allowed_devices[PATH_MAX]);
+static int _read_allowed_devices_file(char **allowed_devices);
 
 extern int task_cgroup_devices_init(void)
 {
@@ -97,7 +101,7 @@ extern int task_cgroup_devices_init(void)
 	cgroup_allowed_devices_file[0] = '\0';
 
 	if (get_procs(&cpunum) != 0) {
-		error("task/cgroup: unable to get a number of CPU");
+		error("unable to get a number of CPU");
 		goto error;
 	}
 
@@ -106,7 +110,7 @@ extern int task_cgroup_devices_init(void)
 	cg_conf = xcgroup_get_slurm_cgroup_conf();
 
 	if ((strlen(cg_conf->allowed_devices_file) + 1) >= PATH_MAX) {
-		error("task/cgroup: device file path length exceeds limit: %s",
+		error("device file path length exceeds limit: %s",
 		      cg_conf->allowed_devices_file);
 		slurm_mutex_unlock(&xcgroup_config_read_mutex);
 		goto error;
@@ -115,13 +119,13 @@ extern int task_cgroup_devices_init(void)
 	slurm_mutex_unlock(&xcgroup_config_read_mutex);
 	if (xcgroup_ns_create(&devices_ns, "", "devices")
 	    != XCGROUP_SUCCESS ) {
-		error("task/cgroup: unable to create devices namespace");
+		error("unable to create devices namespace");
 		goto error;
 	}
 
 	file = fopen(cgroup_allowed_devices_file, "r");
 	if (!file) {
-		debug("task/cgroup: unable to open %s: %m",
+		debug("unable to open %s: %m",
 		      cgroup_allowed_devices_file);
 	} else
 		fclose(file);
@@ -153,21 +157,21 @@ extern int task_cgroup_devices_fini(void)
 					       "devices step");
 
 			if (xcgroup_delete(&step_devices_cg) != SLURM_SUCCESS)
-                                debug2("task/cgroup: unable to remove step "
+                                debug2("unable to remove step "
                                        "devices : %m");
                         if (xcgroup_delete(&job_devices_cg) != XCGROUP_SUCCESS)
-                                debug2("task/cgroup: not removing "
+                                debug2("not removing "
                                        "job devices : %m");
                         if (xcgroup_delete(&user_devices_cg)
 			    != XCGROUP_SUCCESS)
-                                debug2("task/cgroup: not removing "
+                                debug2("not removing "
                                        "user devices : %m");
                         xcgroup_unlock(&devices_cg);
                 } else
-                        error("task/cgroup: unable to lock root devices : %m");
+                        error("unable to lock root devices : %m");
                 xcgroup_destroy(&devices_cg);
         } else
-                error("task/cgroup: unable to create root devices : %m");
+                error("unable to create root devices : %m");
 
 	if ( user_cgroup_path[0] != '\0' )
 		xcgroup_destroy(&user_devices_cg);
@@ -188,134 +192,38 @@ extern int task_cgroup_devices_fini(void)
 	return SLURM_SUCCESS;
 }
 
-extern int task_cgroup_devices_create(stepd_step_rec_t *job)
+static int _handle_device_access(void *x, void *arg)
 {
-	int k, rc, allow_lines = 0;
-	int fstatus = SLURM_ERROR;
-	char *allowed_devices[PATH_MAX], *allowed_dev_major[PATH_MAX];
-	xcgroup_t devices_cg;
-	uint32_t jobid;
-	uint32_t stepid = job->stepid;
-	uid_t uid = job->uid;
-	uid_t gid = job->gid;
+	gres_device_t *gres_device = (gres_device_t *)x;
+	xcgroup_t *devices_cg = (xcgroup_t *)arg;
+	char *cg;
 
+	if (gres_device->alloc)
+		cg = "devices.allow";
+	else
+		cg = "devices.deny";
+
+	log_flag(GRES, "%s %s: adding %s(%s)",
+		 (devices_cg == &job_devices_cg) ? "job " : "step",
+		 cg, gres_device->major, gres_device->path);
+	xcgroup_set_param(devices_cg, cg, gres_device->major);
+
+	return SLURM_SUCCESS;
+}
+
+static int _cgroup_create_callback(const char *calling_func,
+				   xcgroup_ns_t *ns,
+				   void *callback_arg)
+{
+	task_memory_create_callback_t *cgroup_callback =
+		(task_memory_create_callback_t *)callback_arg;
+	stepd_step_rec_t *job = cgroup_callback->job;
+	pid_t pid;
 	List job_gres_list = job->job_gres_list;
 	List step_gres_list = job->step_gres_list;
 	List device_list = NULL;
-	ListIterator itr;
-	gres_device_t *gres_device;
-
-	char* slurm_cgpath ;
-
-	/* create slurm root cgroup in this cgroup namespace */
-	slurm_cgpath = task_cgroup_create_slurm_cg(&devices_ns);
-	if (slurm_cgpath == NULL)
-		return SLURM_ERROR;
-
-	/* build user cgroup relative path if not set (should not be) */
-	if (*user_cgroup_path == '\0') {
-		if (snprintf(user_cgroup_path, PATH_MAX, "%s/uid_%u",
-			     slurm_cgpath, uid) >= PATH_MAX) {
-			error("unable to build uid %u cgroup relative path : %m",
-			      uid);
-			xfree(slurm_cgpath);
-			return SLURM_ERROR;
-		}
-	}
-	xfree(slurm_cgpath);
-
-	/* build job cgroup relative path if no set (should not be) */
-	if (job->het_job_id && (job->het_job_id != NO_VAL))
-		jobid = job->het_job_id;
-	else
-		jobid = job->jobid;
-	if (*job_cgroup_path == '\0') {
-		if (snprintf(job_cgroup_path, PATH_MAX, "%s/job_%u",
-			     user_cgroup_path, jobid) >= PATH_MAX) {
-			error("task/cgroup: unable to build job %u devices "
-			      "cgroup relative path : %m", jobid);
-			return SLURM_ERROR;
-		}
-	}
-
-	/* build job step cgroup relative path (should not be) */
-	if (*jobstep_cgroup_path == '\0') {
-		int cc;
-		if (stepid == SLURM_BATCH_SCRIPT) {
-			cc = snprintf(jobstep_cgroup_path, PATH_MAX,
-				      "%s/step_batch", job_cgroup_path);
-		} else if (stepid == SLURM_EXTERN_CONT) {
-			cc = snprintf(jobstep_cgroup_path, PATH_MAX,
-				      "%s/step_extern", job_cgroup_path);
-		} else {
-			cc = snprintf(jobstep_cgroup_path, PATH_MAX,
-				     "%s/step_%u",
-				     job_cgroup_path, stepid);
-		}
-		if (cc >= PATH_MAX) {
-			error("task/cgroup: unable to build job step %u.%u "
-			      "devices cgroup relative path : %m",
-			      jobid, stepid);
-			return SLURM_ERROR;
-		}
-	}
-
-	/*
-	 * create devices root cgroup and lock it
-	 *
-	 * we will keep the lock until the end to avoid the effect of a release
-	 * agent that would remove an existing cgroup hierarchy while we are
-	 * setting it up. As soon as the step cgroup is created, we can release
-	 * the lock.
-	 * Indeed, consecutive slurm steps could result in cgroup being removed
-	 * between the next EEXIST instantiation and the first addition of
-	 * a task. The release_agent will have to lock the root devices cgroup
-	 * to avoid this scenario.
-	 */
-	if (xcgroup_create(&devices_ns, &devices_cg, "", 0, 0) !=
-	    XCGROUP_SUCCESS ) {
-		error("task/cgroup: unable to create root devices cgroup");
-		return SLURM_ERROR;
-	}
-	if (xcgroup_lock(&devices_cg) != XCGROUP_SUCCESS) {
-		xcgroup_destroy(&devices_cg);
-		error("task/cgroup: unable to lock root devices cgroup");
-		return SLURM_ERROR;
-	}
-
-	debug2("task/cgroup: manage devices jor job '%u'", jobid);
-
-	/*
-	 * create user cgroup in the devices ns (it could already exist)
-	 */
-	if (xcgroup_create(&devices_ns, &user_devices_cg, user_cgroup_path,
-			   getuid(), getgid()) != XCGROUP_SUCCESS) {
-		goto error;
-	}
-	if (xcgroup_instantiate(&user_devices_cg) != XCGROUP_SUCCESS) {
-		xcgroup_destroy(&user_devices_cg);
-		goto error;
-	}
-
-	/* TODO
-	 * check that user's devices cgroup is consistent and allow the
-	 * appropriate devices
-	 */
-
-
-	/*
-	 * create job cgroup in the devices ns (it could already exist)
-	 */
-	if (xcgroup_create(&devices_ns, &job_devices_cg, job_cgroup_path,
-			    getuid(), getgid()) != XCGROUP_SUCCESS) {
-		xcgroup_destroy(&user_devices_cg);
-		goto error;
-	}
-	if (xcgroup_instantiate(&job_devices_cg) != XCGROUP_SUCCESS) {
-		xcgroup_destroy(&user_devices_cg);
-		xcgroup_destroy(&job_devices_cg);
-		goto error;
-	}
+	char *allowed_devices[PATH_MAX], *allowed_dev_major[PATH_MAX];
+	int k, rc, allow_lines = 0;
 
 	/*
          * create the entry with major minor for the default allowed devices
@@ -323,8 +231,6 @@ extern int task_cgroup_devices_create(stepd_step_rec_t *job)
          */
 	allow_lines = _read_allowed_devices_file(allowed_devices);
 	_calc_device_major(allowed_devices, allowed_dev_major, allow_lines);
-	for (k = 0; k < allow_lines; k++)
-		xfree(allowed_devices[k]);
 
 	/*
 	 * with the current cgroup devices subsystem design (whitelist only
@@ -332,8 +238,8 @@ extern int task_cgroup_devices_create(stepd_step_rec_t *job)
 	 * to be allowed by* default.
 	 */
 	for (k = 0; k < allow_lines; k++) {
-		debug2("Default access allowed to device %s for job",
-		       allowed_dev_major[k]);
+		debug2("Default access allowed to device %s(%s) for job",
+		       allowed_dev_major[k], allowed_devices[k]);
 		xcgroup_set_param(&job_devices_cg, "devices.allow",
 				  allowed_dev_major[k]);
 	}
@@ -344,57 +250,22 @@ extern int task_cgroup_devices_create(stepd_step_rec_t *job)
 	device_list = gres_plugin_get_allocated_devices(job_gres_list, true);
 
 	if (device_list) {
-		itr = list_iterator_create(device_list);
-		while ((gres_device = list_next(itr))) {
-			if (gres_device->alloc) {
-				debug("Allowing access to device %s(%s) for job",
-				      gres_device->major, gres_device->path);
-				xcgroup_set_param(&job_devices_cg,
-						  "devices.allow",
-						  gres_device->major);
-			} else {
-				debug("Not allowing access to device %s(%s) for job",
-				       gres_device->major, gres_device->path);
-				xcgroup_set_param(&job_devices_cg,
-						  "devices.deny",
-						  gres_device->major);
-			}
-		}
-		list_iterator_destroy(itr);
-		list_destroy(device_list);
+		list_for_each(device_list, _handle_device_access,
+			      &job_devices_cg);
+		FREE_NULL_LIST(device_list);
 	}
 
-	/*
-	 * create step cgroup in the devices ns (it should not exists)
-	 * use job's user uid/gid to enable tasks cgroups creation by
-	 * the user inside the step cgroup owned by root
-	 */
-	if (xcgroup_create(&devices_ns, &step_devices_cg, jobstep_cgroup_path,
-			   uid, gid) != XCGROUP_SUCCESS ) {
-		/* do not delete user/job cgroup as */
-		/* they can exist for other steps */
-		xcgroup_destroy(&user_devices_cg);
-		xcgroup_destroy(&job_devices_cg);
-		goto error;
-	}
-	if ( xcgroup_instantiate(&step_devices_cg) != XCGROUP_SUCCESS ) {
-		xcgroup_destroy(&user_devices_cg);
-		xcgroup_destroy(&job_devices_cg);
-		xcgroup_destroy(&step_devices_cg);
-		goto error;
-	}
-
-
-	if ((job->stepid != SLURM_BATCH_SCRIPT) &&
-	    (job->stepid != SLURM_EXTERN_CONT)) {
+	if ((job->step_id.step_id != SLURM_BATCH_SCRIPT) &&
+	    (job->step_id.step_id != SLURM_EXTERN_CONT) &&
+	    (job->step_id.step_id != SLURM_INTERACTIVE_STEP)) {
 		/*
 		 * with the current cgroup devices subsystem design (whitelist
 		 * only supported) we need to allow all different devices that
 		 * are supposed to be allowed by default.
 		 */
 		for (k = 0; k < allow_lines; k++) {
-			debug2("Default access allowed to device %s for step",
-			     allowed_dev_major[k]);
+			debug2("Default access allowed to device %s(%s) for step",
+			       allowed_dev_major[k], allowed_devices[k]);
 			xcgroup_set_param(&step_devices_cg, "devices.allow",
 					  allowed_dev_major[k]);
 		}
@@ -407,47 +278,48 @@ extern int task_cgroup_devices_create(stepd_step_rec_t *job)
 			step_gres_list, false);
 
 		if (device_list) {
-			itr = list_iterator_create(device_list);
-			while ((gres_device = list_next(itr))) {
-				if (gres_device->alloc) {
-					debug("Allowing access to device %s(%s) for step",
-					      gres_device->major,
-					      gres_device->path);
-					xcgroup_set_param(&step_devices_cg,
-							  "devices.allow",
-							  gres_device->major);
-				} else {
-					debug("Not allowing access to device %s(%s) for step",
-					      gres_device->major,
-					      gres_device->path);
-					xcgroup_set_param(&step_devices_cg,
-							  "devices.deny",
-							  gres_device->major);
-				}
-			}
-			list_iterator_destroy(itr);
-			list_destroy(device_list);
+			list_for_each(device_list, _handle_device_access,
+				      &step_devices_cg);
+			FREE_NULL_LIST(device_list);
 		}
 	}
-	/* attach the slurmstepd to the step devices cgroup */
-	pid_t pid = getpid();
-	rc = xcgroup_add_pids(&step_devices_cg, &pid, 1);
-	if (rc != XCGROUP_SUCCESS) {
-		error("task/cgroup: unable to add slurmstepd to devices cg '%s'",
-		      step_devices_cg.path);
-		fstatus = SLURM_ERROR;
-	} else {
-		fstatus = SLURM_SUCCESS;
-	}
 
-error:
-	xcgroup_unlock(&devices_cg);
-	xcgroup_destroy(&devices_cg);
 	for (k = 0; k < allow_lines; k++) {
 		xfree(allowed_dev_major[k]);
+		xfree(allowed_devices[k]);
 	}
 
-	return fstatus;
+	/* attach the slurmstepd to the step devices cgroup */
+	pid = getpid();
+	rc = xcgroup_add_pids(&step_devices_cg, &pid, 1);
+	if (rc != XCGROUP_SUCCESS) {
+		error("%s: unable to add slurmstepd to devices cg '%s'",
+		      calling_func, step_devices_cg.path);
+		rc = SLURM_ERROR;
+	} else {
+		rc = SLURM_SUCCESS;
+	}
+
+	return rc;
+}
+
+extern int task_cgroup_devices_create(stepd_step_rec_t *job)
+{
+	task_memory_create_callback_t cgroup_callback = {
+		.job = job,
+	};
+
+	return xcgroup_create_hierarchy(__func__,
+					job,
+					&devices_ns,
+					&job_devices_cg,
+					&step_devices_cg,
+					&user_devices_cg,
+					job_cgroup_path,
+					jobstep_cgroup_path,
+					user_cgroup_path,
+					_cgroup_create_callback,
+					&cgroup_callback);
 }
 
 extern int task_cgroup_devices_attach_task(stepd_step_rec_t *job)
@@ -468,7 +340,7 @@ static void _calc_device_major(char *dev_path[PATH_MAX],
 	int k;
 
 	if (lines > PATH_MAX) {
-		error("task/cgroup: more devices configured than table size "
+		error("more devices configured than table size "
 		      "(%d > %d)", lines, PATH_MAX);
 		lines = PATH_MAX;
 	}
