@@ -144,6 +144,7 @@ typedef struct agent_info {
 	uint16_t retry;			/* if set, keep trying */
 	thd_t *thread_struct;		/* thread structures */
 	bool get_reply;			/* flag if reply expected */
+	uid_t r_uid;			/* receiver UID */
 	slurm_msg_type_t msg_type;	/* RPC to be issued */
 	void **msg_args_pptr;		/* RPC data to be used */
 	uint16_t protocol_version;	/* if set, use this version */
@@ -157,6 +158,7 @@ typedef struct task_info {
 	uint32_t *threads_active_ptr;	/* currently active thread ptr */
 	thd_t *thread_struct_ptr;	/* thread structures ptr */
 	bool get_reply;			/* flag if reply expected */
+	uid_t r_uid;			/* receiver UID */
 	slurm_msg_type_t msg_type;	/* RPC to be issued */
 	void *msg_args_ptr;		/* ptr to RPC data to be used */
 	uint16_t protocol_version;	/* if set, use this version */
@@ -214,6 +216,7 @@ static pthread_mutex_t agent_cnt_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  agent_cnt_cond  = PTHREAD_COND_INITIALIZER;
 static int agent_cnt = 0;
 static int agent_thread_cnt = 0;
+static int mail_thread_cnt = 0;
 static uint16_t message_timeout = NO_VAL16;
 
 static pthread_mutex_t pending_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -259,26 +262,23 @@ void *agent(void *args)
 	}
 #endif
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-		info("%s: Agent_cnt=%d agent_thread_cnt=%d with msg_type=%s retry_list_size=%d",
-		     __func__, agent_cnt, agent_thread_cnt,
-		     rpc_num2string(agent_arg_ptr->msg_type),
-		     retry_list_size());
-	}
+	log_flag(AGENT, "%s: Agent_cnt=%d agent_thread_cnt=%d with msg_type=%s retry_list_size=%d",
+		 __func__, agent_cnt, agent_thread_cnt,
+		 rpc_num2string(agent_arg_ptr->msg_type),
+		 retry_list_size());
+
 	slurm_mutex_lock(&agent_cnt_mutex);
 
-	if (sched_update != slurmctld_conf.last_update) {
+	if (sched_update != slurm_conf.last_update) {
 #ifdef HAVE_NATIVE_CRAY
 		reboot_from_ctld = true;
 #else
-		char *ctld_params = slurm_get_slurmctld_params();
-
 		reboot_from_ctld = false;
-		if (xstrcasestr(ctld_params, "reboot_from_controller"))
+		if (xstrcasestr(slurm_conf.slurmctld_params,
+		                "reboot_from_controller"))
 			reboot_from_ctld = true;
-		xfree(ctld_params);
 #endif
-		sched_update = slurmctld_conf.last_update;
+		sched_update = slurm_conf.last_update;
 	}
 
 	rpc_thread_cnt = 2 + MIN(agent_arg_ptr->node_count, AGENT_THREAD_COUNT);
@@ -314,15 +314,15 @@ void *agent(void *args)
 	/* start the watchdog thread */
 	slurm_thread_create(&thread_wdog, _wdog, agent_info_ptr);
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-		info("%s: New agent thread_count:%d threads_active:%d retry:%c get_reply:%c msg_type:%s protocol_version:%hu",
-		     __func__, agent_info_ptr->thread_count,
-		     agent_info_ptr->threads_active,
-		     agent_info_ptr->retry ? 'T' : 'F',
-		     agent_info_ptr->get_reply ? 'T' : 'F',
-		     rpc_num2string(agent_arg_ptr->msg_type),
-		     agent_info_ptr->protocol_version);
-	}
+	log_flag(AGENT, "%s: New agent thread_count:%d threads_active:%d retry:%c get_reply:%c r_uid:%u msg_type:%s protocol_version:%hu",
+		 __func__, agent_info_ptr->thread_count,
+		 agent_info_ptr->threads_active,
+		 agent_info_ptr->retry ? 'T' : 'F',
+		 agent_info_ptr->get_reply ? 'T' : 'F',
+		 agent_info_ptr->r_uid,
+		 rpc_num2string(agent_arg_ptr->msg_type),
+		 agent_info_ptr->protocol_version);
+
 	/* start all the other threads (up to AGENT_THREAD_COUNT active) */
 	for (i = 0; i < agent_info_ptr->thread_count; i++) {
 		/* wait until "room" for another thread */
@@ -349,7 +349,7 @@ void *agent(void *args)
 	/* Wait for termination of remaining threads */
 	pthread_join(thread_wdog, NULL);
 	delay = (int) difftime(time(NULL), begin_time);
-	if (delay > (slurm_get_msg_timeout() * 2)) {
+	if (delay > (slurm_conf.msg_timeout * 2)) {
 		info("agent msg_type=%u ran for %d seconds",
 			agent_arg_ptr->msg_type,  delay);
 	}
@@ -360,15 +360,13 @@ void *agent(void *args)
 	}
 	slurm_mutex_unlock(&agent_info_ptr->thread_mutex);
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-		info("%s: end agent thread_count:%d threads_active:%d retry:%c get_reply:%c msg_type:%s protocol_version:%hu",
-		     __func__, agent_info_ptr->thread_count,
-		     agent_info_ptr->threads_active,
-		     agent_info_ptr->retry ? 'T' : 'F',
-		     agent_info_ptr->get_reply ? 'T' : 'F',
-		     rpc_num2string(agent_arg_ptr->msg_type),
-		     agent_info_ptr->protocol_version);
-	}
+	log_flag(AGENT, "%s: end agent thread_count:%d threads_active:%d retry:%c get_reply:%c msg_type:%s protocol_version:%hu",
+		 __func__, agent_info_ptr->thread_count,
+		 agent_info_ptr->threads_active,
+		 agent_info_ptr->retry ? 'T' : 'F',
+		 agent_info_ptr->get_reply ? 'T' : 'F',
+		 rpc_num2string(agent_arg_ptr->msg_type),
+		 agent_info_ptr->protocol_version);
 
 cleanup:
 	_purge_agent_args(agent_arg_ptr);
@@ -420,6 +418,11 @@ static int _valid_agent_arg(agent_arg_t *agent_arg_ptr)
 		     __func__, agent_arg_ptr->node_count, hostlist_cnt);
 		return SLURM_ERROR;	/* no messages to be sent */
 	}
+	if (!agent_arg_ptr->r_uid_set) {
+		error("%s: r_uid not set for message:%u ",
+		      __func__, agent_arg_ptr->msg_type);
+		return SLURM_ERROR;
+	}
 	return SLURM_SUCCESS;
 }
 
@@ -442,6 +445,7 @@ static agent_info_t *_make_agent_info(agent_arg_t *agent_arg_ptr)
 	thread_ptr = xcalloc(agent_info_ptr->thread_count, sizeof(thd_t));
 	memset(thread_ptr, 0, (agent_info_ptr->thread_count * sizeof(thd_t)));
 	agent_info_ptr->thread_struct  = thread_ptr;
+	agent_info_ptr->r_uid = agent_arg_ptr->r_uid;
 	agent_info_ptr->msg_type       = agent_arg_ptr->msg_type;
 	agent_info_ptr->msg_args_pptr  = &agent_arg_ptr->msg_args;
 	agent_info_ptr->protocol_version = agent_arg_ptr->protocol_version;
@@ -505,12 +509,9 @@ static agent_info_t *_make_agent_info(agent_arg_t *agent_arg_ptr)
 		thread_ptr[thr_count].nodelist =
 			hostlist_ranged_string_xmalloc(hl);
 		hostlist_destroy(hl);
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-			info("%s: sending msg_type %s to nodes %s",
-			     __func__, rpc_num2string(agent_arg_ptr->msg_type),
-			     thread_ptr[thr_count].nodelist);
-
-		}
+		log_flag(AGENT, "%s: sending msg_type %s to nodes %s",
+			 __func__, rpc_num2string(agent_arg_ptr->msg_type),
+			 thread_ptr[thr_count].nodelist);
 		thr_count++;
 	}
 	xfree(span);
@@ -528,6 +529,7 @@ static task_info_t *_make_task_data(agent_info_t *agent_info_ptr, int inx)
 	task_info_ptr->threads_active_ptr= &agent_info_ptr->threads_active;
 	task_info_ptr->thread_struct_ptr = &agent_info_ptr->thread_struct[inx];
 	task_info_ptr->get_reply         = agent_info_ptr->get_reply;
+	task_info_ptr->r_uid = agent_info_ptr->r_uid;
 	task_info_ptr->msg_type          = agent_info_ptr->msg_type;
 	task_info_ptr->msg_args_ptr      = *agent_info_ptr->msg_args_pptr;
 	task_info_ptr->protocol_version  = agent_info_ptr->protocol_version;
@@ -543,10 +545,8 @@ static void _update_wdog_state(thd_t *thread_ptr,
 	case DSH_ACTIVE:
 		thd_comp->work_done = false;
 		if (thread_ptr->end_time <= thd_comp->now) {
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-				info("%s: agent thread %lu timed out", __func__,
-				     (unsigned long) thread_ptr->thread);
-			}
+			log_flag(AGENT, "%s: agent thread %lu timed out",
+				 __func__, (unsigned long) thread_ptr->thread);
 			if (pthread_kill(thread_ptr->thread, SIGUSR1) == ESRCH)
 				*state = DSH_NO_RESP;
 			else
@@ -650,11 +650,9 @@ static void *_wdog(void *args)
 		xfree(thread_ptr[i].nodelist);
 	}
 
-	if (thd_comp.max_delay &&
-	    (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT)) {
-		info("%s: agent maximum delay %d seconds", __func__,
-		     thd_comp.max_delay);
-	}
+	if (thd_comp.max_delay)
+		log_flag(AGENT, "%s: agent maximum delay %d seconds",
+			 __func__, thd_comp.max_delay);
 
 	slurm_mutex_unlock(&agent_ptr->thread_mutex);
 	return (void *) NULL;
@@ -665,30 +663,30 @@ static void _notify_slurmctld_jobs(agent_info_t *agent_ptr)
 	/* Locks: Write job */
 	slurmctld_lock_t job_write_lock =
 	    { NO_LOCK, WRITE_LOCK, NO_LOCK, NO_LOCK, NO_LOCK };
-	uint32_t job_id = 0, step_id = 0;
+	slurm_step_id_t step_id = {
+		.job_id = 0,
+		.step_id = NO_VAL,
+		.step_het_comp = NO_VAL,
+	};
 	thd_t *thread_ptr = agent_ptr->thread_struct;
 
 	if        (agent_ptr->msg_type == SRUN_PING) {
 		srun_ping_msg_t *msg = *agent_ptr->msg_args_pptr;
-		job_id  = msg->job_id;
-		step_id = msg->step_id;
+		step_id.job_id  = msg->job_id;
 	} else if (agent_ptr->msg_type == SRUN_TIMEOUT) {
 		srun_timeout_msg_t *msg = *agent_ptr->msg_args_pptr;
-		job_id  = msg->job_id;
-		step_id = msg->step_id;
+		memcpy(&step_id, &msg->step_id, sizeof(step_id));
 	} else if (agent_ptr->msg_type == RESPONSE_RESOURCE_ALLOCATION) {
 		resource_allocation_response_msg_t *msg =
 			*agent_ptr->msg_args_pptr;
-		job_id  = msg->job_id;
-		step_id = NO_VAL;
+		step_id.job_id = msg->job_id;
 	} else if (agent_ptr->msg_type == RESPONSE_HET_JOB_ALLOCATION) {
 		List het_alloc_list = *agent_ptr->msg_args_pptr;
 		resource_allocation_response_msg_t *msg;
 		if (!het_alloc_list || (list_count(het_alloc_list) == 0))
 			return;
 		msg = list_peek(het_alloc_list);
-		job_id  = msg->job_id;
-		step_id = NO_VAL;
+		step_id.job_id  = msg->job_id;
 	} else if ((agent_ptr->msg_type == SRUN_JOB_COMPLETE)		||
 		   (agent_ptr->msg_type == SRUN_REQUEST_SUSPEND)	||
 		   (agent_ptr->msg_type == SRUN_STEP_MISSING)		||
@@ -704,7 +702,7 @@ static void _notify_slurmctld_jobs(agent_info_t *agent_ptr)
 	}
 	lock_slurmctld(job_write_lock);
 	if  (thread_ptr[0].state == DSH_DONE) {
-		srun_response(job_id, step_id);
+		srun_response(&step_id);
 	}
 
 	unlock_slurmctld(job_write_lock);
@@ -738,7 +736,7 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 				  .fed  = READ_LOCK };
 
 			lock_slurmctld(job_write_lock);
-			job_complete(job_id, slurmctld_conf.slurm_user_id,
+			job_complete(job_id, slurm_conf.slurm_user_id,
 				     true, false, 0);
 			unlock_slurmctld(job_write_lock);
 		}
@@ -779,9 +777,8 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 #ifdef HAVE_FRONT_END
 				down_msg = "";
 #else
-				drain_nodes(node_names,
-					    "Prolog/Epilog failure",
-					    slurmctld_conf.slurm_user_id);
+				drain_nodes(node_names, "Prolog/Epilog failure",
+				            slurm_conf.slurm_user_id);
 				down_msg = ", set to state DRAIN";
 #endif
 				error("Prolog/Epilog failure on nodes %s%s",
@@ -791,9 +788,8 @@ static void _notify_slurmctld_nodes(agent_info_t *agent_ptr,
 #ifdef HAVE_FRONT_END
 				down_msg = "";
 #else
-				drain_nodes(node_names,
-					    "Duplicate jobid",
-					    slurmctld_conf.slurm_user_id);
+				drain_nodes(node_names, "Duplicate jobid",
+				            slurm_conf.slurm_user_id);
 				down_msg = ", set to state DRAIN";
 #endif
 				error("Duplicate jobid on nodes %s%s",
@@ -930,11 +926,10 @@ static void *_thread_per_group_rpc(void *args)
 
 	msg.msg_type = msg_type;
 	msg.data     = task_ptr->msg_args_ptr;
+	slurm_msg_set_r_uid(&msg, task_ptr->r_uid);
 
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-		info("%s: sending %s to %s", __func__, rpc_num2string(msg_type),
-		     thread_ptr->nodelist);
-	}
+	log_flag(AGENT, "%s: sending %s to %s",
+		 __func__, rpc_num2string(msg_type), thread_ptr->nodelist);
 
 	if (task_ptr->get_reply) {
 		if (thread_ptr->addr) {
@@ -1019,7 +1014,7 @@ static void *_thread_per_group_rpc(void *args)
 				task_ptr->msg_args_ptr;
 			rc = SLURM_SUCCESS;
 			lock_slurmctld(job_write_lock);
-			if (job_epilog_complete(kill_job->job_id,
+			if (job_epilog_complete(kill_job->step_id.job_id,
 						ret_data_info->node_name, rc))
 				run_scheduler = true;
 			unlock_slurmctld(job_write_lock);
@@ -1047,8 +1042,8 @@ static void *_thread_per_group_rpc(void *args)
 			thread_state = DSH_DONE;
 			ret_data_info->err = thread_state;
 			lock_slurmctld(job_write_lock);
-			job_complete(job_id, slurmctld_conf.slurm_user_id,
-				     false, false, _wif_status());
+			job_complete(job_id, slurm_conf.slurm_user_id,
+			             false, false, _wif_status());
 			unlock_slurmctld(job_write_lock);
 			continue;
 		} else if ((msg_type == RESPONSE_RESOURCE_ALLOCATION) &&
@@ -1063,8 +1058,8 @@ static void *_thread_per_group_rpc(void *args)
 			     job_id, slurm_strerror(rc));
 			thread_state = DSH_FAILED;
 			lock_slurmctld(job_write_lock);
-			job_complete(job_id, slurmctld_conf.slurm_user_id,
-				     false, false, _wif_status());
+			job_complete(job_id, slurm_conf.slurm_user_id,
+			             false, false, _wif_status());
 			unlock_slurmctld(job_write_lock);
 			continue;
 		} else if ((msg_type == RESPONSE_HET_JOB_ALLOCATION) &&
@@ -1083,8 +1078,8 @@ static void *_thread_per_group_rpc(void *args)
 			     job_id, slurm_strerror(rc));
 			thread_state = DSH_FAILED;
 			lock_slurmctld(job_write_lock);
-			job_complete(job_id, slurmctld_conf.slurm_user_id,
-				     false, false, _wif_status());
+			job_complete(job_id, slurm_conf.slurm_user_id,
+			             false, false, _wif_status());
 			unlock_slurmctld(job_write_lock);
 			continue;
 		}
@@ -1096,7 +1091,7 @@ static void *_thread_per_group_rpc(void *args)
 
 			if ((msg_ptr->signal == SIGCONT) ||
 			    (msg_ptr->signal == SIGSTOP)) {
-				job_id = msg_ptr->job_id;
+				job_id = msg_ptr->step_id.job_id;
 				lock_slurmctld(job_write_lock);
 				job_ptr = find_job_record(job_id);
 				if (job_ptr == NULL) {
@@ -1156,10 +1151,8 @@ static void *_thread_per_group_rpc(void *args)
 			/* Not indicative of a real error */
 		case ESLURMD_JOB_NOTRUNNING:
 			/* Not indicative of a real error */
-			if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-				info("%s: RPC to node %s failed, job not running",
-				     __func__, ret_data_info->node_name);
-			}
+			log_flag(AGENT, "%s: RPC to node %s failed, job not running",
+				 __func__, ret_data_info->node_name);
 			thread_state = DSH_DONE;
 			break;
 		default:
@@ -1199,7 +1192,7 @@ cleanup:
 			task_ptr->msg_args_ptr;
 		if ((msg_ptr->signal == SIGCONT) ||
 		    (msg_ptr->signal == SIGSTOP)) {
-			job_id = msg_ptr->job_id;
+			job_id = msg_ptr->step_id.job_id;
 			lock_slurmctld(job_write_lock);
 			job_ptr = find_job_record(job_id);
 			if (job_ptr)
@@ -1243,8 +1236,8 @@ static int _setup_requeue(agent_arg_t *agent_arg_ptr, thd_t *thread_ptr,
 
 	itr = list_iterator_create(thread_ptr->ret_list);
 	while ((ret_data_info = list_next(itr))) {
-		if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT)
-			info("%s: got err of %d", __func__, ret_data_info->err);
+		log_flag(AGENT, "%s: got err of %d",
+			 __func__, ret_data_info->err);
 		if (ret_data_info->err != DSH_NO_RESP)
 			continue;
 
@@ -1303,6 +1296,8 @@ static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count)
 	agent_arg_ptr->msg_args = *(agent_info_ptr->msg_args_pptr);
 	*(agent_info_ptr->msg_args_pptr) = NULL;
 
+	set_agent_arg_r_uid(agent_arg_ptr, agent_info_ptr->r_uid);
+
 	j = 0;
 	for (i = 0; i < agent_info_ptr->thread_count; i++) {
 		if (!thread_ptr[i].ret_list) {
@@ -1442,14 +1437,10 @@ extern void agent_init(void)
  */
 extern void agent_trigger(int min_wait, bool mail_too)
 {
-	if (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT) {
-		info("%s: pending_wait_time=%d->%d mail_too=%c->%c Agent_cnt=%d agent_thread_cnt=%d retry_list_size=%d",
-		     __func__, pending_wait_time, min_wait,
-		     mail_too ?  'T' : 'F',
-		     pending_mail ? 'T' : 'F',
-		     agent_cnt, agent_thread_cnt,
-		     retry_list_size());
-	}
+	log_flag(AGENT, "%s: pending_wait_time=%d->%d mail_too=%c->%c Agent_cnt=%d agent_thread_cnt=%d retry_list_size=%d",
+		 __func__, pending_wait_time, min_wait,
+		 mail_too ?  'T' : 'F', pending_mail ? 'T' : 'F',
+		 agent_cnt, agent_thread_cnt, retry_list_size());
 
 	slurm_mutex_lock(&pending_mutex);
 	if ((pending_wait_time == NO_VAL16) ||
@@ -1608,7 +1599,7 @@ static void _agent_retry(int min_wait, bool mail_too)
 		if (((list_size > 100) &&
 		     (difftime(now, last_msg_time) > 300)) ||
 		    ((list_size > 0) &&
-		     (slurmctld_conf.debug_flags & DEBUG_FLAG_AGENT))) {
+		     (slurm_conf.debug_flags & DEBUG_FLAG_AGENT))) {
 			/* Note sizable backlog (retry_list_size()) of work */
 			retry_iter = list_iterator_create(retry_list);
 			while ((queued_req_ptr = list_next(retry_iter))) {
@@ -1679,11 +1670,13 @@ static void _agent_retry(int min_wait, bool mail_too)
 	} else if (mail_too) {
 		slurm_mutex_lock(&agent_cnt_mutex);
 		slurm_mutex_lock(&mail_mutex);
-		while (mail_list && (agent_thread_cnt < MAX_SERVER_THREADS)) {
+		while (mail_list && (agent_thread_cnt < MAX_SERVER_THREADS) &&
+		       (mail_thread_cnt < MAX_MAIL_THREADS)) {
 			mi = (mail_info_t *) list_dequeue(mail_list);
 			if (!mi)
 				break;
 
+			mail_thread_cnt++;
 			agent_thread_cnt++;
 			slurm_thread_create_detached(NULL, _mail_proc, mi);
 		}
@@ -1707,7 +1700,7 @@ void agent_queue_request(agent_arg_t *agent_arg_ptr)
 		fatal("AGENT_THREAD_COUNT value is too high relative to MAX_SERVER_THREADS");
 
 	if (message_timeout == NO_VAL16) {
-		message_timeout = MAX(slurm_get_msg_timeout(), 30);
+		message_timeout = MAX(slurm_conf.msg_timeout, 30);
 	}
 
 	if (agent_arg_ptr->msg_type == REQUEST_SHUTDOWN) {
@@ -1866,16 +1859,13 @@ static void _mail_free(void *arg)
 
 static char **_build_mail_env(void)
 {
-	char **my_env;
+	char **my_env = xcalloc(2, sizeof(char *));
 
-        my_env = xmalloc(sizeof(char *));
-        my_env[0] = NULL;
-        if (slurmctld_conf.cluster_name) {
-                setenvf(&my_env, "SLURM_CLUSTER_NAME", "%s",
-                        slurmctld_conf.cluster_name);
-        }
+	my_env[0] = xstrdup_printf("SLURM_CLUSTER_NAME=%s",
+	                           slurm_conf.cluster_name);
+	my_env[1] = NULL;
 
-	 return my_env;
+	return my_env;
 }
 
 /* process an email request and free the record */
@@ -1899,21 +1889,25 @@ static void *_mail_proc(void *arg)
 			error("Couldn't do a dup on fd 1: %m");
 		if ((fd_2 = dup(fd_0)) == -1)			// fd = 2
 			error("Couldn't do a dup on fd 2 %m");
-		execle(slurmctld_conf.mail_prog, "mail",
-			"-s", mi->message, mi->user_name,
-			NULL, my_env);
-		error("Failed to exec %s: %m",
-			slurmctld_conf.mail_prog);
-		exit(1);
+		execle(slurm_conf.mail_prog, "mail", "-s", mi->message,
+		       mi->user_name, NULL, my_env);
+		error("Failed to exec %s: %m", slurm_conf.mail_prog);
+		_exit(1);
 	} else {		/* parent */
 		waitpid(pid, NULL, 0);
 	}
 	_mail_free(mi);
 	slurm_mutex_lock(&agent_cnt_mutex);
+	slurm_mutex_lock(&mail_mutex);
 	if (agent_thread_cnt)
 		agent_thread_cnt--;
 	else
 		error("agent_thread_cnt underflow");
+	if (mail_thread_cnt)
+		mail_thread_cnt--;
+	else
+		error("mail_thread_cnt underflow");
+	slurm_mutex_unlock(&mail_mutex);
 	slurm_mutex_unlock(&agent_cnt_mutex);
 
 	return (void *) NULL;
@@ -1921,6 +1915,8 @@ static void *_mail_proc(void *arg)
 
 static char *_mail_type_str(uint16_t mail_type)
 {
+	if (mail_type == MAIL_INVALID_DEPEND)
+		return "Invalid dependency";
 	if (mail_type == MAIL_JOB_BEGIN)
 		return "Began";
 	if (mail_type == MAIL_JOB_END)
@@ -2154,7 +2150,8 @@ static int _batch_launch_defer(queued_request_t *queued_req_ptr)
 
 	if (job_ptr->wait_all_nodes) {
 		(void) job_node_ready(launch_msg_ptr->job_id, &tmp);
-		if (tmp == (READY_JOB_STATE | READY_NODE_STATE)) {
+		if (tmp ==
+		    (READY_JOB_STATE | READY_NODE_STATE | READY_PROLOG_STATE)) {
 			nodes_ready = 1;
 			if (launch_msg_ptr->alias_list &&
 			    !xstrcmp(launch_msg_ptr->alias_list, "TBD")) {
@@ -2200,7 +2197,7 @@ static int _batch_launch_defer(queued_request_t *queued_req_ptr)
 		queued_req_ptr->first_attempt = now;
 		queued_req_ptr->last_attempt  = now;
 	} else if (difftime(now, queued_req_ptr->first_attempt) >=
-				 slurm_get_resume_timeout()) {
+		   slurm_conf.resume_timeout) {
 		/* Nodes will get marked DOWN and job requeued, if possible */
 		error("agent waited too long for nodes to respond, abort launch of JobId=%u",
 		      job_ptr->job_id);
@@ -2225,11 +2222,11 @@ static int _signal_defer(queued_request_t *queued_req_ptr)
 
 	agent_arg_ptr = queued_req_ptr->agent_arg_ptr;
 	signal_msg_ptr = (signal_tasks_msg_t *)agent_arg_ptr->msg_args;
-	job_ptr = find_job_record(signal_msg_ptr->job_id);
+	job_ptr = find_job_record(signal_msg_ptr->step_id.job_id);
 
 	if (job_ptr == NULL) {
 		info("agent(signal_task): removed pending request for cancelled JobId=%u",
-		     signal_msg_ptr->job_id);
+		     signal_msg_ptr->step_id.job_id);
 		return -1;	/* job cancelled while waiting */
 	}
 
@@ -2239,7 +2236,7 @@ static int _signal_defer(queued_request_t *queued_req_ptr)
 	if (queued_req_ptr->first_attempt == 0) {
 		queued_req_ptr->first_attempt = now;
 	} else if (difftime(now, queued_req_ptr->first_attempt) >=
-				 2 * slurm_get_batch_start_timeout()) {
+	           (2 * slurm_conf.batch_start_timeout)) {
 		error("agent waited too long for nodes to respond, abort signal of JobId=%u",
 		      job_ptr->job_id);
 		return -1;
@@ -2266,16 +2263,16 @@ static void _reboot_from_ctld(agent_arg_t *agent_arg_ptr)
 		error("%s: hostlist is NULL", __func__);
 		return;
 	}
-	if (!slurmctld_conf.reboot_program) {
+	if (!slurm_conf.reboot_program) {
 		error("%s: RebootProgram is NULL", __func__);
 		return;
 	}
 
-	pname = strrchr(slurmctld_conf.reboot_program, '/');
+	pname = strrchr(slurm_conf.reboot_program, '/');
 	if (pname)
 		argv[0] = pname + 1;
 	else
-		argv[0] = slurmctld_conf.reboot_program;
+		argv[0] = slurm_conf.reboot_program;
 	argv[1] = hostlist_deranged_string_xmalloc(agent_arg_ptr->hostlist);
 	argv[2] = NULL;
 
@@ -2284,8 +2281,8 @@ static void _reboot_from_ctld(agent_arg_t *agent_arg_ptr)
 		for (i = 0; i < 1024; i++)
 			(void) close(i);
 		(void) setpgid(0, 0);
-		(void) execv(slurmctld_conf.reboot_program, argv);
-		exit(1);
+		(void) execv(slurm_conf.reboot_program, argv);
+		_exit(1);
 	} else if (child < 0) {
 		error("fork: %m");
 	} else {
@@ -2303,3 +2300,10 @@ static void _reboot_from_ctld(agent_arg_t *agent_arg_ptr)
 	}
 	xfree(argv[1]);
 }
+
+/* Set r_uid of agent_arg */
+extern void set_agent_arg_r_uid(agent_arg_t *agent_arg_ptr, uid_t r_uid)
+{
+	agent_arg_ptr->r_uid = r_uid;
+	agent_arg_ptr->r_uid_set = true;
+}
