@@ -76,6 +76,7 @@
 #include "src/slurmd/slurmstepd/step_terminate_monitor.h"
 
 #include "src/slurmd/common/task_plugin.h"
+#include "src/slurmd/common/job_container_plugin.h"
 
 static void *_handle_accept(void *arg);
 static int _handle_request(int fd, stepd_step_rec_t *job,
@@ -104,6 +105,7 @@ static int _handle_stat_jobacct(int fd, stepd_step_rec_t *job, uid_t uid);
 static int _handle_task_info(int fd, stepd_step_rec_t *job);
 static int _handle_list_pids(int fd, stepd_step_rec_t *job);
 static int _handle_reconfig(int fd, stepd_step_rec_t *job, uid_t uid);
+static int _handle_get_ns_fd(int fd, stepd_step_rec_t *job);
 static bool _msg_socket_readable(eio_obj_t *obj);
 static int _msg_socket_accept(eio_obj_t *obj, List objs);
 
@@ -139,7 +141,7 @@ static int msg_target_node_id = 0;
 static bool
 _slurm_authorized_user(uid_t uid)
 {
-	return ((uid == (uid_t) 0) || (uid == conf->slurm_user_id));
+	return ((uid == (uid_t) 0) || (uid == slurm_conf.slurm_user_id));
 }
 
 /*
@@ -190,7 +192,7 @@ _create_socket(const char *name)
 
 static int
 _domain_socket_create(const char *dir, const char *nodename,
-		     uint32_t jobid, uint32_t stepid)
+		      slurm_step_id_t *step_id)
 {
 	int fd;
 	char *name = NULL;
@@ -210,7 +212,11 @@ _domain_socket_create(const char *dir, const char *nodename,
 	/*
 	 * Now build the name of socket, and create the socket.
 	 */
-	xstrfmtcat(name, "%s/%s_%u.%u", dir, nodename, jobid, stepid);
+	xstrfmtcat(name, "%s/%s_%u.%u", dir, nodename, step_id->job_id,
+		   step_id->step_id);
+
+	if (step_id->step_het_comp != NO_VAL)
+		xstrfmtcat(name, ".%u", step_id->step_het_comp);
 
 	/*
 	 * First check to see if the named socket already exists.
@@ -221,8 +227,8 @@ _domain_socket_create(const char *dir, const char *nodename,
 		 * and recreate it.
 		 */
 		if (unlink(name) != 0) {
-			error("%s: failed unlink(%s) job %u step %u %m",
-			      __func__, name, jobid, stepid);
+			error("%s: failed unlink(%s): %m",
+			      __func__, name);
 			xfree(name);
 			errno = ESLURMD_STEP_EXISTS;
 			return -1;
@@ -269,8 +275,8 @@ static int _wait_for_job_running(stepd_step_rec_t *job)
 	}
 
 	if (job->state < SLURMSTEPD_STEP_RUNNING) {
-		debug("step %u.%u not running yet %d [cont_id:%"PRIu64"]",
-		      job->jobid, job->stepid, job->state, job->cont_id);
+		debug("%ps not running yet %d [cont_id:%"PRIu64"]",
+		      &job->step_id, job->state, job->cont_id);
 		rc = ESLURMD_JOB_NOTRUNNING;
 	}
 
@@ -297,7 +303,7 @@ msg_thr_create(stepd_step_rec_t *job)
 	eio_obj_t *eio_obj;
 	errno = 0;
 	fd = _domain_socket_create(conf->spooldir, conf->node_name,
-				   job->jobid, job->stepid);
+				   &job->step_id);
 	if (fd == -1)
 		return SLURM_ERROR;
 
@@ -421,48 +427,8 @@ static void *_handle_accept(void *arg)
 	debug3("%s: entering (new thread)", __func__);
 	xfree(arg);
 
-	/*
-	 * Prior to 19.05, REQUEST_CONNECT was the first thing written on
-	 * the socket, followed by an auth_cred. In 19.05, this changes over
-	 * to SLURM_PROTOCOL_VERSION from the client and the (unneeded)
-	 * credential is no longer received. The REQUEST_CONNECT block should
-	 * be removed two versions after 19.05.
-	 */
 	safe_read(fd, &req, sizeof(int));
-	if (req == REQUEST_CONNECT) {
-		int len;
-		void *auth_cred;
-		char *auth_info = slurm_get_auth_info();
-
-		/* assume oldest we can talk to */
-		client_protocol_ver = SLURM_MIN_PROTOCOL_VERSION;
-
-		safe_read(fd, &len, sizeof(int));
-		buffer = init_buf(len);
-		safe_read(fd, get_buf_data(buffer), len);
-
-		/* Unpack and verify the auth credential */
-		auth_cred = g_slurm_auth_unpack(buffer, SLURM_PROTOCOL_VERSION);
-		if (auth_cred == NULL) {
-			error("Unpacking authentication credential: %m");
-			FREE_NULL_BUFFER(buffer);
-			goto fail;
-		}
-		rc = g_slurm_auth_verify(auth_cred, auth_info);
-		if (rc != SLURM_SUCCESS) {
-			error("Verifying authentication credential: %m");
-			xfree(auth_info);
-			(void) g_slurm_auth_destroy(auth_cred);
-			FREE_NULL_BUFFER(buffer);
-			goto fail;
-		}
-		xfree(auth_info);
-
-		/* Get the uid from the credential, then destroy it. */
-		uid = g_slurm_auth_get_uid(auth_cred);
-		g_slurm_auth_destroy(auth_cred);
-		FREE_NULL_BUFFER(buffer);
-	} else if (req >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (req >= SLURM_MIN_PROTOCOL_VERSION) {
 #if defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__)
 		gid_t tmp_gid;
 
@@ -621,6 +587,10 @@ int _handle_request(int fd, stepd_step_rec_t *job, uid_t uid, pid_t remote_pid)
 		debug("Handling REQUEST_GETGR");
 		rc = _handle_getgr(fd, job, remote_pid);
 		break;
+	case REQUEST_GET_NS_FD:
+		debug("Handling REQUEST_GET_NS_FD");
+		rc = _handle_get_ns_fd(fd, job);
+		break;
 	default:
 		error("Unrecognized request: %d", req);
 		rc = SLURM_ERROR;
@@ -647,8 +617,8 @@ _handle_info(int fd, stepd_step_rec_t *job)
 	uint16_t protocol_version = SLURM_PROTOCOL_VERSION;
 
 	safe_write(fd, &job->uid, sizeof(uid_t));
-	safe_write(fd, &job->jobid, sizeof(uint32_t));
-	safe_write(fd, &job->stepid, sizeof(uint32_t));
+	safe_write(fd, &job->step_id.job_id, sizeof(uint32_t));
+	safe_write(fd, &job->step_id.step_id, sizeof(uint32_t));
 
 	/* protocol_version was added in Slurm version 2.2,
 	 * so it needed to be added later in the data sent
@@ -659,6 +629,7 @@ _handle_info(int fd, stepd_step_rec_t *job)
 	safe_write(fd, &job->nodeid, sizeof(uint32_t));
 	safe_write(fd, &job->job_mem, sizeof(uint64_t));
 	safe_write(fd, &job->step_mem, sizeof(uint64_t));
+	safe_write(fd, &job->step_id.step_het_comp, sizeof(uint32_t));
 
 	return SLURM_SUCCESS;
 rwfail:
@@ -711,13 +682,13 @@ _handle_signal_container(int fd, stepd_step_rec_t *job, uid_t uid)
 	safe_read(fd, &flag, sizeof(int));
 	safe_read(fd, &req_uid, sizeof(uid_t));
 
-	debug("_handle_signal_container for step=%u.%u uid=%d signal=%d",
-	      job->jobid, job->stepid, (int) req_uid, sig);
+	debug("_handle_signal_container for %ps uid=%d signal=%d",
+	      &job->step_id, (int) req_uid, sig);
 	/* verify uid off uid instead of req_uid as we can trust that one */
 	if ((uid != job->uid) && !_slurm_authorized_user(uid)) {
-		error("signal container req from uid %ld for step=%u.%u "
+		error("signal container req from uid %ld for %ps "
 		      "owned by uid %ld",
-		      (long)req_uid, job->jobid, job->stepid, (long)job->uid);
+		      (long)req_uid, &job->step_id, (long)job->uid);
 		rc = -1;
 		errnum = EPERM;
 		goto done;
@@ -751,17 +722,20 @@ _handle_signal_container(int fd, stepd_step_rec_t *job, uid_t uid)
 		}
 	}
 
-	if ((job->stepid != SLURM_EXTERN_CONT) &&
+	if ((job->step_id.step_id != SLURM_EXTERN_CONT) &&
 	    (job->nodeid == msg_target_node_id) && (msg_sent == 0) &&
 	    (job->state < SLURMSTEPD_STEP_ENDING)) {
 		time_t now = time(NULL);
-		char entity[24], time_str[24];
+		char entity[45], time_str[24];
 
-		if (job->stepid == SLURM_BATCH_SCRIPT) {
-			snprintf(entity, sizeof(entity), "JOB %u", job->jobid);
+		if (job->step_id.step_id == SLURM_BATCH_SCRIPT) {
+			snprintf(entity, sizeof(entity), "JOB %u", job->step_id.job_id);
 		} else {
-			snprintf(entity, sizeof(entity), "STEP %u.%u",
-				 job->jobid, job->stepid);
+			char tmp_char[33];
+			log_build_step_id_str(&job->step_id, tmp_char,
+					      sizeof(tmp_char),
+					      STEP_ID_FLAG_NO_PREFIX);
+			snprintf(entity, sizeof(entity), "STEP %s", tmp_char);
 		}
 		slurm_make_time_str(&now, time_str, sizeof(time_str));
 
@@ -828,17 +802,16 @@ _handle_signal_container(int fd, stepd_step_rec_t *job, uid_t uid)
 	}
 
 	if (sig == SIG_TERM_KILL) {
-		uint16_t kill_wait = slurm_get_kill_wait();
 		(void) proctrack_g_signal(job->cont_id, SIGCONT);
 		(void) proctrack_g_signal(job->cont_id, SIGTERM);
-		sleep(kill_wait);
+		sleep(slurm_conf.kill_wait);
 		sig = SIGKILL;
 	}
 
 	/*
 	 * Specific handle for the batch container and some related flags.
 	 */
-	if (job->stepid == SLURM_BATCH_SCRIPT &&
+	if (job->step_id.step_id == SLURM_BATCH_SCRIPT &&
 	    ((flag & KILL_JOB_BATCH) || (flag & KILL_FULL_JOB))) {
 
 		if (flag & KILL_FULL_JOB)
@@ -846,18 +819,16 @@ _handle_signal_container(int fd, stepd_step_rec_t *job, uid_t uid)
 		else
 			rc = kill(job->pgid, sig);
 		if (rc < 0) {
-			error("%s: failed signal %d pid %u job %u.%u %m",
-			      __func__, sig, job->pgid,
-			      job->jobid, job->stepid);
+			error("%s: failed signal %d pid %u %ps %m",
+			      __func__, sig, job->pgid, &job->step_id);
 			rc = SLURM_ERROR;
 			errnum = errno;
 			slurm_mutex_unlock(&suspend_mutex);
 			goto done;
 		}
 
-		verbose("%s: sent signal %d to pid %u job %u.%u",
-			__func__, sig, job->pgid,
-			job->jobid, job->stepid);
+		verbose("%s: sent signal %d to pid %u %ps",
+			__func__, sig, job->pgid, &job->step_id);
 		rc = SLURM_SUCCESS;
 		errnum = 0;
 		slurm_mutex_unlock(&suspend_mutex);
@@ -870,11 +841,10 @@ _handle_signal_container(int fd, stepd_step_rec_t *job, uid_t uid)
 	if (proctrack_g_signal(job->cont_id, sig) < 0) {
 		rc = -1;
 		errnum = errno;
-		verbose("Error sending signal %d to %u.%u: %m",
-			sig, job->jobid, job->stepid);
+		verbose("Error sending signal %d to %ps: %m",
+			sig, &job->step_id);
 	} else {
-		verbose("Sent signal %d to %u.%u",
-			sig, job->jobid, job->stepid);
+		verbose("Sent signal %d to %ps", sig, &job->step_id);
 	}
 	slurm_mutex_unlock(&suspend_mutex);
 
@@ -894,8 +864,7 @@ _handle_notify_job(int fd, stepd_step_rec_t *job, uid_t uid)
 	int len;
 	char *message = NULL;
 
-	debug3("_handle_notify_job for job %u.%u",
-	       job->jobid, job->stepid);
+	debug3("_handle_notify_job for %ps", &job->step_id);
 
 	safe_read(fd, &len, sizeof(int));
 	if (len) {
@@ -905,9 +874,9 @@ _handle_notify_job(int fd, stepd_step_rec_t *job, uid_t uid)
 
 	debug3("  uid = %d", uid);
 	if ((uid != job->uid) && !_slurm_authorized_user(uid)) {
-		debug("notify req from uid %ld for job %u.%u "
+		debug("notify req from uid %ld for %ps "
 		      "owned by uid %ld",
-		      (long)uid, job->jobid, job->stepid, (long)job->uid);
+		      (long)uid, &job->step_id, (long)job->uid);
 		rc = EPERM;
 		goto done;
 	}
@@ -934,15 +903,14 @@ _handle_terminate(int fd, stepd_step_rec_t *job, uid_t uid)
 	uint32_t i;
 
 	if (uid != job->uid && !_slurm_authorized_user(uid)) {
-		debug("terminate req from uid %ld for job %u.%u "
+		debug("terminate req from uid %ld for %ps "
 		      "owned by uid %ld",
-		      (long)uid, job->jobid, job->stepid, (long)job->uid);
+		      (long)uid, &job->step_id, (long)job->uid);
 		rc = -1;
 		errnum = EPERM;
 		goto done;
 	}
-	debug("_handle_terminate for step=%u.%u uid=%d",
-	      job->jobid, job->stepid, uid);
+	debug("_handle_terminate for %ps uid=%d", &job->step_id, uid);
 	step_terminate_monitor_start(job);
 
 	/*
@@ -976,8 +944,7 @@ _handle_terminate(int fd, stepd_step_rec_t *job, uid_t uid)
 	 */
 	slurm_mutex_lock(&suspend_mutex);
 	if (suspended) {
-		debug("Terminating suspended job step %u.%u",
-		      job->jobid, job->stepid);
+		debug("Terminating suspended %ps", &job->step_id);
 		suspended = false;
 	}
 
@@ -986,11 +953,10 @@ _handle_terminate(int fd, stepd_step_rec_t *job, uid_t uid)
 			rc = -1;
 			errnum = errno;
 		}
-		verbose("Error sending SIGKILL signal to %u.%u: %m",
-			job->jobid, job->stepid);
+		verbose("Error sending SIGKILL signal to %ps: %m",
+			&job->step_id);
 	} else {
-		verbose("Sent SIGKILL signal to %u.%u",
-			job->jobid, job->stepid);
+		verbose("Sent SIGKILL signal to %ps", &job->step_id);
 	}
 	slurm_mutex_unlock(&suspend_mutex);
 
@@ -1011,7 +977,7 @@ _handle_attach(int fd, stepd_step_rec_t *job, uid_t uid)
 	uint32_t *gtids = NULL, *pids = NULL;
 	int len, i;
 
-	debug("_handle_attach for job %u.%u", job->jobid, job->stepid);
+	debug("_handle_attach for %ps", &job->step_id);
 
 	srun       = xmalloc(sizeof(srun_info_t));
 	srun->key  = (srun_key_t *)xmalloc(SLURM_IO_KEY_SIZE);
@@ -1021,6 +987,7 @@ _handle_attach(int fd, stepd_step_rec_t *job, uid_t uid)
 	safe_read(fd, &srun->ioaddr, sizeof(slurm_addr_t));
 	safe_read(fd, &srun->resp_addr, sizeof(slurm_addr_t));
 	safe_read(fd, srun->key, SLURM_IO_KEY_SIZE);
+	safe_read(fd, &srun->uid, sizeof(uid_t));
 	safe_read(fd, &srun->protocol_version, sizeof(uint16_t));
 
 	if (!srun->protocol_version)
@@ -1039,8 +1006,8 @@ _handle_attach(int fd, stepd_step_rec_t *job, uid_t uid)
 	 * call, so only _slurm_authorized_user is allowed.
 	 */
 	if (!_slurm_authorized_user(uid)) {
-		error("uid %ld attempt to attach to job %u.%u owned by %ld",
-		      (long) uid, job->jobid, job->stepid, (long)job->uid);
+		error("uid %ld attempt to attach to %ps owned by %ld",
+		      (long) uid, &job->step_id, (long)job->uid);
 		rc = EPERM;
 		goto done;
 	}
@@ -1111,8 +1078,7 @@ _handle_pid_in_container(int fd, stepd_step_rec_t *job)
 	bool rc = false;
 	pid_t pid;
 
-	debug("_handle_pid_in_container for job %u.%u",
-	      job->jobid, job->stepid);
+	debug("_handle_pid_in_container for %ps", &job->step_id);
 
 	safe_read(fd, &pid, sizeof(pid_t));
 
@@ -1127,6 +1093,32 @@ rwfail:
 	return SLURM_ERROR;
 }
 
+static int _handle_get_ns_fd(int fd, stepd_step_rec_t *job)
+{
+	int ns_fd = -1;
+
+	debug("%s: for job %u:%u",
+	      __func__, job->step_id.job_id, job->step_id.step_id);
+
+	ns_fd = container_g_join_external(job->step_id.job_id);
+
+	/*
+	 * We need to send the ns_fd as an int first to let the receiver know if
+	 * we have a valid fd or not as recv_fd_over_pipe() will always try to
+	 * set up the fd no matter if it is valid or not.
+	 */
+	safe_write(fd, &ns_fd, sizeof(ns_fd));
+	if (ns_fd > 0)
+		send_fd_over_pipe(fd, ns_fd);
+
+	debug("sent fd: %d", ns_fd);
+	debug("leaving %s", __func__);
+
+	return SLURM_SUCCESS;
+rwfail:
+	return SLURM_ERROR;
+}
+
 static void _block_on_pid(pid_t pid)
 {
 	/* I wish there was another way to wait on a foreign pid, but
@@ -1210,14 +1202,13 @@ static int _handle_add_extern_pid_internal(stepd_step_rec_t *job, pid_t pid)
 	extern_pid_t *extern_pid;
 	jobacct_id_t jobacct_id;
 
-	if (job->stepid != SLURM_EXTERN_CONT) {
+	if (job->step_id.step_id != SLURM_EXTERN_CONT) {
 		error("%s: non-extern step (%u) given for job %u.",
-		      __func__, job->stepid, job->jobid);
+		      __func__, job->step_id.step_id, job->step_id.job_id);
 		return SLURM_ERROR;
 	}
 
-	debug("%s: for job %u.%u, pid %d",
-	      __func__, job->jobid, job->stepid, pid);
+	debug("%s: for %ps, pid %d", __func__, &job->step_id, pid);
 
 	extern_pid = xmalloc(sizeof(extern_pid_t));
 	extern_pid->job = job;
@@ -1231,17 +1222,20 @@ static int _handle_add_extern_pid_internal(stepd_step_rec_t *job, pid_t pid)
 	jobacct_id.job = job;
 
 	if (proctrack_g_add(job, pid) != SLURM_SUCCESS) {
-		error("%s: Job %u can't add pid %d to proctrack plugin in the extern_step.", __func__, job->jobid, pid);
+		error("%s: Job %u can't add pid %d to proctrack plugin in the extern_step.",
+		      __func__, job->step_id.job_id, pid);
 		return SLURM_ERROR;
 	}
 
 	if (task_g_add_pid(pid) != SLURM_SUCCESS) {
-		error("%s: Job %u can't add pid %d to task plugin in the extern_step.", __func__, job->jobid, pid);
+		error("%s: Job %u can't add pid %d to task plugin in the extern_step.",
+		      __func__, job->step_id.job_id, pid);
 		return SLURM_ERROR;
 	}
 
 	if (jobacct_gather_add_task(pid, &jobacct_id, 1) != SLURM_SUCCESS) {
-		error("%s: Job %u can't add pid %d to jobacct_gather plugin in the extern_step.", __func__, job->jobid, pid);
+		error("%s: Job %u can't add pid %d to jobacct_gather plugin in the extern_step.",
+		      __func__, job->step_id.job_id, pid);
 		return SLURM_ERROR;
 	}
 
@@ -1468,12 +1462,12 @@ _handle_suspend(int fd, stepd_step_rec_t *job, uid_t uid)
 
 	safe_read(fd, &job_core_spec, sizeof(uint16_t));
 
-	debug("_handle_suspend for step:%u.%u uid:%ld core_spec:%u",
-	      job->jobid, job->stepid, (long)uid, job_core_spec);
+	debug("_handle_suspend for %ps uid:%ld core_spec:%u",
+	      &job->step_id, (long)uid, job_core_spec);
 
 	if (!_slurm_authorized_user(uid)) {
-		debug("job step suspend request from uid %ld for job %u.%u ",
-		      (long)uid, job->jobid, job->stepid);
+		debug("job step suspend request from uid %ld for %ps ",
+		      (long)uid, &job->step_id);
 		rc = -1;
 		errnum = EPERM;
 		goto done;
@@ -1509,16 +1503,16 @@ _handle_suspend(int fd, stepd_step_rec_t *job, uid_t uid)
 		 * also permit longer time periods when more than one job can
 		 * be running on each resource (not good). */
 		if (proctrack_g_signal(job->cont_id, SIGTSTP) < 0) {
-			verbose("Error suspending %u.%u (SIGTSTP): %m",
-				job->jobid, job->stepid);
+			verbose("Error suspending %ps (SIGTSTP): %m",
+				&job->step_id);
 		} else
 			sleep(2);
 
 		if (proctrack_g_signal(job->cont_id, SIGSTOP) < 0) {
-			verbose("Error suspending %u.%u (SIGSTOP): %m",
-				job->jobid, job->stepid);
+			verbose("Error suspending %ps (SIGSTOP): %m",
+				&job->step_id);
 		} else {
-			verbose("Suspended %u.%u", job->jobid, job->stepid);
+			verbose("Suspended %ps", &job->step_id);
 		}
 		suspended = true;
 	}
@@ -1547,12 +1541,12 @@ _handle_resume(int fd, stepd_step_rec_t *job, uid_t uid)
 
 	safe_read(fd, &job_core_spec, sizeof(uint16_t));
 
-	debug("_handle_resume for step:%u.%u uid:%ld core_spec:%u",
-	      job->jobid, job->stepid, (long)uid, job_core_spec);
+	debug("_handle_resume for %ps uid:%ld core_spec:%u",
+	      &job->step_id, (long)uid, job_core_spec);
 
 	if (!_slurm_authorized_user(uid)) {
-		debug("job step resume request from uid %ld for job %u.%u ",
-		      (long)uid, job->jobid, job->stepid);
+		debug("job step resume request from uid %ld for %ps ",
+		      (long)uid, &job->step_id);
 		rc = -1;
 		errnum = EPERM;
 		goto done;
@@ -1580,10 +1574,9 @@ _handle_resume(int fd, stepd_step_rec_t *job, uid_t uid)
 						      job_core_spec))
 			error("core_spec_g_resume: %m");
 		if (proctrack_g_signal(job->cont_id, SIGCONT) < 0) {
-			verbose("Error resuming %u.%u: %m",
-				job->jobid, job->stepid);
+			verbose("Error resuming %ps: %m", &job->step_id);
 		} else {
-			verbose("Resumed %u.%u", job->jobid, job->stepid);
+			verbose("Resumed %ps", &job->step_id);
 		}
 		suspended = false;
 	}
@@ -1623,13 +1616,12 @@ _handle_completion(int fd, stepd_step_rec_t *job, uid_t uid)
 	Buf buffer = NULL;
 	bool lock_set = false;
 
-	debug("_handle_completion for job %u.%u",
-	      job->jobid, job->stepid);
+	debug("_handle_completion for %ps", &job->step_id);
 
 	debug3("  uid = %d", uid);
 	if (!_slurm_authorized_user(uid)) {
-		debug("step completion message from uid %ld for job %u.%u ",
-		      (long)uid, job->jobid, job->stepid);
+		debug("step completion message from uid %ld for %ps ",
+		      (long)uid, &job->step_id);
 		rc = -1;
 		errnum = EPERM;
 		/* Send the return code and errno */
@@ -1678,17 +1670,34 @@ _handle_completion(int fd, stepd_step_rec_t *job, uid_t uid)
 	 *  without the hostlist from the credential.
 	 */
 	if (step_complete.bits && (step_complete.rank >= 0)) {
+		int32_t set_bits;
+		int32_t first_bit = first - (step_complete.rank + 1);
+		int32_t last_bit = last - (step_complete.rank + 1);
+		/* bit_set_count_range is [first, end) so +1 last_bit */
+		int32_t last_bit_range = last_bit + 1;
+
 #if 0
 		char bits_string[128];
 		debug2("Setting range %d (bit %d) through %d(bit %d)",
-		       first, first-(step_complete.rank+1),
-		       last, last-(step_complete.rank+1));
+		       first, first_bit,
+		       last, last_bit);
 		bit_fmt(bits_string, sizeof(bits_string), step_complete.bits);
 		debug2("  before bits: %s", bits_string);
 #endif
-		bit_nset(step_complete.bits,
-			 first - (step_complete.rank+1),
-			 last - (step_complete.rank+1));
+		if (!(set_bits = bit_set_count_range(step_complete.bits,
+						     first_bit,
+						     last_bit_range))) {
+			bit_nset(step_complete.bits, first_bit, last_bit);
+		} else if (set_bits == (last_bit_range - first_bit)) {
+			debug("Step complete from %d to %d was already processed on rank %d. Probably a RPC was resent from a child.",
+			      first, last, step_complete.rank);
+			goto timeout;
+		} else {
+			error("Step complete from %d to %d was half-way processed on rank %d. This should never happen.",
+			      first, last, step_complete.rank);
+			goto timeout;
+		}
+
 #if 0
 		bit_fmt(bits_string, sizeof(bits_string), step_complete.bits);
 		debug2("  after bits: %s", bits_string);
@@ -1731,14 +1740,13 @@ _handle_stat_jobacct(int fd, stepd_step_rec_t *job, uid_t uid)
 	jobacctinfo_t *temp_jobacct = NULL;
 	int i = 0;
 	int num_tasks = 0;
-	debug("_handle_stat_jobacct for job %u.%u",
-	      job->jobid, job->stepid);
+	debug("_handle_stat_jobacct for %ps", &job->step_id);
 
 	debug3("  uid = %d", uid);
 	if (uid != job->uid && !_slurm_authorized_user(uid)) {
-		debug("stat jobacct from uid %ld for job %u.%u "
+		debug("stat jobacct from uid %ld for %ps "
 		      "owned by uid %ld",
-		      (long)uid, job->jobid, job->stepid, (long)job->uid);
+		      (long)uid, &job->step_id, (long)job->uid);
 		/* Send NULL */
 		jobacctinfo_setinfo(jobacct, JOBACCT_DATA_PIPE, &fd,
 				    SLURM_PROTOCOL_VERSION);
@@ -1777,7 +1785,7 @@ _handle_task_info(int fd, stepd_step_rec_t *job)
 	int i;
 	stepd_step_task_info_t *task;
 
-	debug("_handle_task_info for job %u.%u", job->jobid, job->stepid);
+	debug("_handle_task_info for %ps", &job->step_id);
 
 	safe_write(fd, &job->node_tasks, sizeof(uint32_t));
 	for (i = 0; i < job->node_tasks; i++) {
@@ -1803,7 +1811,7 @@ _handle_list_pids(int fd, stepd_step_rec_t *job)
 	int npids = 0;
 	uint32_t pid;
 
-	debug("_handle_list_pids for job %u.%u", job->jobid, job->stepid);
+	debug("_handle_list_pids for %ps", &job->step_id);
 	proctrack_g_get_pids(job->cont_id, &pids, &npids);
 	safe_write(fd, &npids, sizeof(uint32_t));
 	for (i = 0; i < npids; i++) {
@@ -1827,9 +1835,8 @@ _handle_reconfig(int fd, stepd_step_rec_t *job, uid_t uid)
 	int errnum = 0;
 
 	if (!_slurm_authorized_user(uid)) {
-		debug("job step reconfigure request from uid %ld "
-		      "for job %u.%u ",
-		      (long)uid, job->jobid, job->stepid);
+		debug("job step reconfigure request from uid %ld for %ps",
+		      (long)uid, &job->step_id);
 		rc = -1;
 		errnum = EPERM;
 		goto done;
@@ -1841,8 +1848,7 @@ _handle_reconfig(int fd, stepd_step_rec_t *job, uid_t uid)
 	 * current fd incorrect.
 	 */
 	log_alter(conf->log_opts, SYSLOG_FACILITY_DAEMON, conf->logfile);
-	debug("_handle_reconfigure for job %u.%u successful",
-	      job->jobid, job->stepid);
+	debug("_handle_reconfigure for %ps successful", &job->step_id);
 
 done:
 	/* Send the return code and errno */
