@@ -43,7 +43,6 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <sys/mman.h>
-#include <sys/stat.h>
 #include <unistd.h>
 
 #include "src/common/assoc_mgr.h"
@@ -76,21 +75,20 @@
 #include "src/slurmd/slurmstepd/slurmstepd.h"
 #include "src/slurmd/slurmstepd/slurmstepd_job.h"
 
-static int _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
+static int _init_from_slurmd(int sock, char **argv,
+			     slurm_addr_t **_cli, uid_t *_cli_uid,
 			     slurm_addr_t **_self, slurm_msg_t **_msg);
 
 static void _dump_user_env(void);
 static void _send_ok_to_slurmd(int sock);
 static void _send_fail_to_slurmd(int sock);
 static void _got_ack_from_slurmd(int);
-static stepd_step_rec_t *_step_setup(slurm_addr_t *cli, slurm_addr_t *self,
-				     slurm_msg_t *msg);
+static stepd_step_rec_t *_step_setup(slurm_addr_t *cli, uid_t cli_uid,
+				     slurm_addr_t *self, slurm_msg_t *msg);
 #ifdef MEMORY_LEAK_DEBUG
 static void _step_cleanup(stepd_step_rec_t *job, slurm_msg_t *msg, int rc);
 #endif
 static int _process_cmdline (int argc, char **argv);
-static void _remove_spool_job_dir(stepd_step_rec_t *job);
-static void _remove_spool_sub_dir(const char *directory);
 
 /*
  *  List of signals to block in this process
@@ -110,6 +108,7 @@ main (int argc, char **argv)
 {
 	log_options_t lopts = LOG_OPTS_INITIALIZER;
 	slurm_addr_t *cli;
+	uid_t cli_uid;
 	slurm_addr_t *self;
 	slurm_msg_t *msg;
 	stepd_step_rec_t *job;
@@ -134,11 +133,11 @@ main (int argc, char **argv)
 		fatal( "failed to initialize authentication plugin" );
 
 	/* Receive job parameters from the slurmd */
-	_init_from_slurmd(STDIN_FILENO, argv, &cli, &self, &msg);
+	_init_from_slurmd(STDIN_FILENO, argv, &cli, &cli_uid, &self, &msg);
 
 	/* Create the stepd_step_rec_t, mostly from info in a
 	 * launch_tasks_request_msg_t or a batch_job_launch_msg_t */
-	if (!(job = _step_setup(cli, self, msg))) {
+	if (!(job = _step_setup(cli, cli_uid, self, msg))) {
 		_send_fail_to_slurmd(STDOUT_FILENO);
 		rc = SLURM_ERROR;
 		goto ending;
@@ -205,8 +204,6 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *job,
 	if (conf->hwloc_xml)
 		(void)remove(conf->hwloc_xml);
 
-	_remove_spool_job_dir(job);
-
 #ifdef MEMORY_LEAK_DEBUG
 	acct_gather_conf_destroy();
 	(void) core_spec_g_fini();
@@ -506,7 +503,8 @@ static void _set_job_log_prefix(slurm_step_id_t *step_id)
  */
 static int
 _init_from_slurmd(int sock, char **argv,
-		  slurm_addr_t **_cli, slurm_addr_t **_self, slurm_msg_t **_msg)
+		  slurm_addr_t **_cli, uid_t *_cli_uid, slurm_addr_t **_self,
+		  slurm_msg_t **_msg)
 {
 	char *incoming_buffer = NULL;
 	Buf buffer;
@@ -514,6 +512,7 @@ _init_from_slurmd(int sock, char **argv,
 	int len;
 	uint16_t proto;
 	slurm_addr_t *cli = NULL;
+	uid_t cli_uid;
 	slurm_addr_t *self = NULL;
 	slurm_msg_t *msg = NULL;
 	slurm_step_id_t step_id = {
@@ -565,6 +564,7 @@ _init_from_slurmd(int sock, char **argv,
 	if (slurm_unpack_addr_no_alloc(cli, buffer) == SLURM_ERROR)
 		fatal("slurmstepd: problem with unpack of slurmd_conf");
 	free_buf(buffer);
+	safe_read(sock, &cli_uid, sizeof(uid_t));
 
 	/* receive self from slurmd */
 	safe_read(sock, &len, sizeof(int));
@@ -665,6 +665,7 @@ _init_from_slurmd(int sock, char **argv,
 	msg->protocol_version = proto;
 
 	*_cli = cli;
+	*_cli_uid = cli_uid;
 	*_self = self;
 	*_msg = msg;
 
@@ -676,7 +677,8 @@ rwfail:
 }
 
 static stepd_step_rec_t *
-_step_setup(slurm_addr_t *cli, slurm_addr_t *self, slurm_msg_t *msg)
+_step_setup(slurm_addr_t *cli, uid_t cli_uid, slurm_addr_t *self,
+	    slurm_msg_t *msg)
 {
 	stepd_step_rec_t *job = NULL;
 
@@ -687,7 +689,7 @@ _step_setup(slurm_addr_t *cli, slurm_addr_t *self, slurm_msg_t *msg)
 		break;
 	case REQUEST_LAUNCH_TASKS:
 		debug2("setup for a launch_task");
-		job = mgr_launch_tasks_setup(msg->data, cli, self,
+		job = mgr_launch_tasks_setup(msg->data, cli, cli_uid, self,
 					     msg->protocol_version);
 		break;
 	default:
@@ -711,7 +713,7 @@ _step_setup(slurm_addr_t *cli, slurm_addr_t *self, slurm_msg_t *msg)
 					   job->step_id.job_id,
 					   job->step_id.step_id);
 	}
-	if (msg->msg_type == REQUEST_BATCH_JOB_LAUNCH) {
+	if (job->batch || (job->step_id.step_id == SLURM_INTERACTIVE_STEP)) {
 		gres_plugin_job_set_env(&job->env, job->job_gres_list, 0);
 	} else if (msg->msg_type == REQUEST_LAUNCH_TASKS) {
 		gres_plugin_step_set_env(&job->env, job->step_gres_list, 0,
@@ -771,79 +773,3 @@ static void _dump_user_env(void)
 	for (i=0; environ[i]; i++)
 		printf("%s\n",environ[i]);
 }
-
-static void
-_remove_spool_job_dir(stepd_step_rec_t *job)
-{
-	char *job_id = NULL, *directory = NULL;
-
-	if (!job->env)
-		return;
-	job_id = xstrdup(getenvp(job->env, "SLURM_JOB_ID"));
-	if (job_id) {
-		xstrfmtcat(directory, "%s/%s", conf->spooldir, job_id);
-		debug("%s: conf->spooldir: %s", __func__, conf->spooldir);
-		debug("%s: job_id: %s", __func__, job_id);
-		debug("%s: directory: %s", __func__, directory);
-		_remove_spool_sub_dir(directory);
-		xfree(directory);
-		xfree(job_id);
-	}
-}
-
-static void
-_remove_spool_sub_dir(const char *directory)
-{
-	const char *spool = "/var/spool/";
-	char *strdir = NULL, *startp = NULL, *prefix = NULL;
-	size_t len1 = 0, len2 = 0;
-	struct stat stat_buf;
-	DIR *dp;
-	struct dirent *ent;
-	char *pathname = NULL;
-
-	if (!directory)
-		return;
-
-	/*
-	 * Don't remove to any directory other than slurmd spool sub directory.
-	 * (The default value is "/var/spool/slurmd/<sub_dir>[/<sub_dir>...]".)
-	 */
-	if (xstrncmp(conf->spooldir, spool, strlen(spool)) != 0)
-		return;
-	len1 = strlen(conf->spooldir);
-	if (xstrncmp(directory, conf->spooldir, len1) != 0)
-		return;
-	strdir = xstrdup(directory);
-	startp = (strdir + len1);
-	while ((prefix = xstrchr(startp, '/'))) {
-		len2 += 1;
-		startp = (prefix + 1);
-	}
-	xfree(strdir);
-	if (strlen(directory) <= (len1 + len2))
-		return;
-
-	if (stat(directory, &stat_buf) < 0)
-		return;
-	if (!S_ISDIR(stat_buf.st_mode))
-		return;
-	if ((dp = opendir(directory)) == NULL)
-		return;
-	while ((ent = readdir(dp)) != NULL) {
-		if (xstrcmp(ent->d_name, ".") == 0)
-			continue;
-		if (xstrcmp(ent->d_name, "..") == 0)
-			continue;
-		xstrfmtcat(pathname, "%s/%s", directory, ent->d_name);
-		stat(pathname, &stat_buf);
-		if (S_ISDIR(stat_buf.st_mode)) {
-			if (rmdir(pathname) < 0)
-				_remove_spool_sub_dir(pathname);
-		} else
-			(void)unlink(pathname);
-		xfree(pathname);
-	}
-	closedir(dp);
-	(void)rmdir(directory);
-}
