@@ -95,8 +95,7 @@ slurm_opt_t opt =
 List 	opt_list = NULL;
 int	pass_number = 0;
 time_t	srun_begin_time = 0;
-bool	tres_bind_err_log = true;
-bool	tres_freq_err_log = true;
+bool local_het_step = false;
 
 /*---- forward declarations of static variables and functions  ----*/
 
@@ -283,15 +282,15 @@ static slurm_opt_t *_opt_copy(void)
 	opt_dup->efname = xstrdup(opt.efname);
 	opt_dup->srun_opt->epilog = xstrdup(sropt.epilog);
 	opt_dup->exclude = xstrdup(opt.exclude);
-	opt_dup->srun_opt->export_env = xstrdup(sropt.export_env);
+	opt_dup->export_env = xstrdup(opt.export_env);
 	opt_dup->extra = xstrdup(opt.extra);
 	opt.gres = NULL;		/* Moved by memcpy */
-	opt_dup->gpu_bind = NULL;	/* Moved by memcpy */
-	opt_dup->gpu_freq = NULL;	/* Moved by memcpy */
-	opt_dup->gpus = NULL;		/* Moved by memcpy */
-	opt_dup->gpus_per_node = NULL;	/* Moved by memcpy */
-	opt_dup->gpus_per_socket = NULL;/* Moved by memcpy */
-	opt_dup->gpus_per_task = NULL;	/* Moved by memcpy */
+	opt.gpu_bind = NULL;		/* Moved by memcpy */
+	opt.gpu_freq = NULL;		/* Moved by memcpy */
+	opt.gpus = NULL;		/* Moved by memcpy */
+	opt.gpus_per_node = NULL;	/* Moved by memcpy */
+	opt.gpus_per_socket = NULL;	/* Moved by memcpy */
+	opt.gpus_per_task = NULL;	/* Moved by memcpy */
 	opt_dup->ifname = xstrdup(opt.ifname);
 	opt_dup->job_name = xstrdup(opt.job_name);
 	opt.licenses = NULL;		/* Moved by memcpy */
@@ -337,6 +336,7 @@ extern int initialize_and_process_args(int argc, char **argv, int *argc_off)
 	bitstr_t *het_grp_bits;
 	int i, i_first, i_last;
 	bool opt_found = false;
+	static bool check_het_step = false;
 
 	het_grp_bits = _get_het_group(argc, argv, default_het_job_offset++,
 				      &opt_found);
@@ -369,7 +369,7 @@ extern int initialize_and_process_args(int argc, char **argv, int *argc_off)
 		/* do not set adjust defaults in an active allocation */
 		if (!getenv("SLURM_JOB_ID")) {
 			bool first = (pass_number == 1);
-			if (cli_filter_plugin_setup_defaults(&opt, first)) {
+			if (cli_filter_g_setup_defaults(&opt, first)) {
 				error("cli_filter plugin terminated with error");
 				exit(error_exit);
 			}
@@ -391,7 +391,49 @@ extern int initialize_and_process_args(int argc, char **argv, int *argc_off)
 		if (argc_off)
 			*argc_off = optind;
 
-		if (cli_filter_plugin_pre_submit(&opt, i)) {
+		if (!check_het_step) {
+			/*
+			 * SLURM_HET_SIZE not defined for a normal allocation.
+			 * SLURM_JOB_ID defined if allocation already exists.
+			 *
+			 * Here we are seeing if we are trying to run a het step
+			 * in the normal allocation.  If so and we didn't
+			 * request nodes on the command line we will clear this
+			 * env variable and figure it out later instead of
+			 * trying to use the whole allocation.
+			 */
+			if (!getenv("SLURM_HET_SIZE") &&
+			    getenv("SLURM_JOB_ID") &&
+			    (optind >= 0) && (optind < argc)) {
+				for (int i2 = optind; i2 < argc; i2++) {
+					if (!xstrcmp(argv[i2], ":")) {
+						local_het_step = true;
+						break;
+					}
+				}
+			}
+			check_het_step = true;
+
+			if (local_het_step) {
+				/*
+				 * If we are a het step don't just unset the
+				 * JOB_NUM_NODES to avoid using it.
+				 */
+				unsetenv("SLURM_JOB_NUM_NODES");
+
+				/*
+				 * If we already set the nodes based off this
+				 * env var reset it.
+				 */
+				if (slurm_option_set_by_env(&opt, 'N')) {
+					opt.nodes_set = false;
+					opt.min_nodes = 1;
+					opt.max_nodes = 0;
+				}
+			}
+		}
+
+		if (cli_filter_g_pre_submit(&opt, i)) {
 			error("cli_filter plugin terminated with error");
 			exit(error_exit);
 		}
@@ -458,6 +500,7 @@ static void _opt_default(void)
 	 * of the job/step. Do not use xfree() as the pointers have been copied.
 	 * See initialize_and_process_args() above.
 	 */
+	sropt.exclusive = true;
 	opt.job_flags			= 0;
 	sropt.multi_prog_cmds		= 0;
 	sropt.het_group			= NULL;
@@ -503,6 +546,7 @@ env_vars_t env_vars[] = {
   { "SLURM_DISABLE_STATUS", 'X' },
   { "SLURM_DISTRIBUTION", 'm' },
   { "SLURM_EPILOG", LONG_OPT_EPILOG },
+  { "SLURM_EXACT", LONG_OPT_EXACT },
   { "SLURM_EXCLUSIVE", LONG_OPT_EXCLUSIVE },
   { "SLURM_EXPORT_ENV", LONG_OPT_EXPORT },
   { "SRUN_EXPORT_ENV", LONG_OPT_EXPORT }, /* overrides SLURM_EXPORT_ENV */
@@ -534,8 +578,11 @@ env_vars_t env_vars[] = {
   { "SLURM_NTASKS", 'n' },
   { "SLURM_NSOCKETS_PER_NODE", LONG_OPT_SOCKETSPERNODE },
   { "SLURM_NTASKS_PER_NODE", LONG_OPT_NTASKSPERNODE },
+  { "SLURM_NTASKS_PER_GPU", LONG_OPT_NTASKSPERGPU },
+  { "SLURM_NTASKS_PER_TRES", LONG_OPT_NTASKSPERTRES },
   { "SLURM_OPEN_MODE", LONG_OPT_OPEN_MODE },
   { "SLURM_OVERCOMMIT", 'O' },
+  { "SLURM_OVERLAP", LONG_OPT_OVERLAP },
   { "SLURM_PARTITION", 'p' },
   { "SLURM_POWER", LONG_OPT_POWER },
   { "SLURM_PROFILE", LONG_OPT_PROFILE },
@@ -555,6 +602,7 @@ env_vars_t env_vars[] = {
   { "SLURM_TASK_PROLOG", LONG_OPT_TASK_PROLOG },
   { "SLURM_THREAD_SPEC", LONG_OPT_THREAD_SPEC },
   { "SLURM_THREADS", 'T' },
+  { "SLURM_THREADS_PER_CORE", LONG_OPT_THREADSPERCORE },
   { "SLURM_TIMELIMIT", 't' },
   { "SLURM_UNBUFFEREDIO", 'u' },
   { "SLURM_USE_MIN_NODES", LONG_OPT_USE_MIN_NODES },
@@ -579,7 +627,8 @@ static void _opt_env(int het_job_offset)
 
 	while (e->var) {
 		if ((val = getenv(e->var)))
-			slurm_process_option(&opt, e->type, val, true, false);
+			slurm_process_option_or_exit(&opt, e->type, val, true,
+						     false);
 		if ((het_job_offset >= 0) &&
 		    strcmp(e->var, "SLURM_JOBID") &&
 		    strcmp(e->var, "SLURM_JOB_ID")) {
@@ -587,25 +636,17 @@ static void _opt_env(int het_job_offset)
 			snprintf(key, sizeof(key), "%s_PACK_GROUP_%d",
 				 e->var, het_job_offset);
 			if ((val = getenv(key)))
-				slurm_process_option(&opt, e->type, val,
-						     true, false);
+				slurm_process_option_or_exit(&opt, e->type, val,
+							     true, false);
 			snprintf(key, sizeof(key), "%s_HET_GROUP_%d",
 				 e->var, het_job_offset);
 			if ((val = getenv(key)))
-				slurm_process_option(&opt, e->type, val,
-						     true, false);
+				slurm_process_option_or_exit(&opt, e->type, val,
+							     true, false);
 		}
 		e++;
 	}
 
-	/* Running srun within an existing srun. Don't inherit values. */
-	if (getenv("SLURM_STEP_ID")) {
-		xfree(sropt.cpu_bind);
-		sropt.cpu_bind_type = 0;
-		xfree(opt.mem_bind);
-		opt.mem_bind_type = 0;
-	}
-
 	/* Process spank env options */
 	if (spank_process_env_options())
 		exit(error_exit);
@@ -633,7 +674,8 @@ static bitstr_t *_get_het_group(const int argc, char **argv,
 	optind = 0;
 	while ((opt_char = getopt_long(argc, argv, opt_string,
 				       optz, &option_index)) != -1) {
-		slurm_process_option(&opt, opt_char, optarg, false, true);
+		slurm_process_option_or_exit(&opt, opt_char, optarg, false,
+					     true);
 	}
 	slurm_option_table_destroy(optz);
 	xfree(opt_string);
@@ -686,7 +728,8 @@ static void _set_options(const int argc, char **argv)
 	optind = 0;
 	while ((opt_char = getopt_long(argc, argv, opt_string,
 				       optz, &option_index)) != -1) {
-		slurm_process_option(&opt, opt_char, optarg, false, false);
+		slurm_process_option_or_exit(&opt, opt_char, optarg, false,
+					     false);
 	}
 
 	slurm_option_table_destroy(optz);
@@ -700,7 +743,7 @@ static void _opt_args(int argc, char **argv, int het_job_offset)
 {
 	int i, command_pos = 0, command_args = 0;
 	char **rest = NULL;
-	char *fullpath, *launch_params;
+	char *fullpath;
 
 	sropt.het_grp_bits = bit_alloc(MAX_HET_JOB_COMPONENTS);
 	bit_set(sropt.het_grp_bits, het_job_offset);
@@ -752,14 +795,9 @@ static void _opt_args(int argc, char **argv, int het_job_offset)
 	}
 	sropt.argv[i] = NULL;	/* End of argv's (for possible execv) */
 
-	if (getenv("SLURM_TEST_EXEC")) {
+	if (getenv("SLURM_TEST_EXEC") ||
+	    xstrstr(slurm_conf.launch_params, "test_exec"))
 		sropt.test_exec = true;
-	} else {
-		launch_params = slurm_get_launch_params();
-		if (launch_params && strstr(launch_params, "test_exec"))
-			sropt.test_exec = true;
-		xfree(launch_params);
-	}
 
 	if (sropt.test_exec) {
 		/* Validate command's existence */
@@ -823,6 +861,8 @@ static bool _opt_verify(void)
 	hostlist_t hl = NULL;
 	int hl_cnt = 0;
 
+	validate_options_salloc_sbatch_srun(&opt);
+
 	/*
 	 *  Do not set slurmd debug level higher than DEBUG2,
 	 *   as DEBUG3 is used for slurmd IO operations, which
@@ -855,6 +895,11 @@ static bool _opt_verify(void)
 		xfree(opt.burst_buffer_file);
 	}
 
+	if (sropt.exact && sropt.whole) {
+		error("--exact and --whole are mutually exclusive.");
+		verified = false;
+	}
+
 	if (sropt.no_alloc && !opt.nodelist) {
 		error("must specify a node list with -Z, --no-allocate.");
 		verified = false;
@@ -877,9 +922,9 @@ static bool _opt_verify(void)
 	}
 
 	if (!sropt.epilog)
-		sropt.epilog = slurm_get_srun_epilog();
+		sropt.epilog = xstrdup(slurm_conf.srun_epilog);
 	if (!sropt.prolog)
-		sropt.prolog = slurm_get_srun_prolog();
+		sropt.prolog = xstrdup(slurm_conf.srun_prolog);
 
 	/*
 	 * This means --ntasks was read from the environment.
@@ -893,16 +938,21 @@ static bool _opt_verify(void)
 	}
 
 	if (opt.hint &&
-	    (!(sropt.cpu_bind_type & ~CPU_BIND_VERBOSE)) &&
-	    (opt.ntasks_per_core == NO_VAL) &&
-	    (opt.threads_per_core == NO_VAL)) {
-		if (verify_hint(opt.hint,
-				&opt.sockets_per_node,
-				&opt.cores_per_socket,
-				&opt.threads_per_core,
-				&opt.ntasks_per_core,
-				&sropt.cpu_bind_type)) {
-			exit(error_exit);
+	    !validate_hint_option(&opt)) {
+		if (sropt.cpu_bind_type & ~CPU_BIND_VERBOSE) {
+			if (opt.verbose)
+				info("--hint and --cpu-bind (other than --cpu-bind=verbose) are mutually exclusive. Ignoring --hint.");
+		} else {
+			xassert(opt.ntasks_per_core == NO_VAL);
+			xassert(opt.threads_per_core == NO_VAL);
+			if (verify_hint(opt.hint,
+					&opt.sockets_per_node,
+					&opt.cores_per_socket,
+					&opt.threads_per_core,
+					&opt.ntasks_per_core,
+					&sropt.cpu_bind_type)) {
+				exit(error_exit);
+			}
 		}
 	}
 
@@ -979,10 +1029,10 @@ static bool _opt_verify(void)
 
 	if (sropt.parallel_debug) {
 		/* Set --threads 1 */
-		slurm_process_option(&opt, 'T', "1", false, false);
+		slurm_process_option_or_exit(&opt, 'T', "1", false, false);
 		/* Set --msg-timeout 15 */
-		slurm_process_option(&opt, LONG_OPT_MSG_TIMEOUT, "1",
-				     false, false);
+		slurm_process_option_or_exit(&opt, LONG_OPT_MSG_TIMEOUT, "1",
+					     false, false);
 	}
 
 	pmi_server_max_threads(sropt.max_threads);
@@ -1151,7 +1201,8 @@ static bool _opt_verify(void)
 			 * which influences future decisions.
 			 */
 			xstrfmtcat(tmp, "%d", opt.min_nodes);
-			slurm_process_option(&opt, 'N', tmp, false, false);
+			slurm_process_option_or_exit(&opt, 'N', tmp, false,
+						     false);
 			xfree(tmp);
 			if (hl_cnt > opt.min_nodes) {
 				int del_cnt, i;
@@ -1176,7 +1227,7 @@ static bool _opt_verify(void)
 				     "requested nodes %u. Ignoring "
 				     "--ntasks-per-node.", opt.ntasks_per_node,
 				     opt.ntasks, opt.min_nodes);
-			opt.ntasks_per_node = NO_VAL;
+			slurm_option_reset(&opt, "ntasks-per-node");
 		}
 
 	} /* else if (opt.ntasks_set && !opt.nodes_set) */
@@ -1195,7 +1246,7 @@ static bool _opt_verify(void)
 	}
 
 	if (!sropt.mpi_type)
-		sropt.mpi_type = slurm_get_mpi_default();
+		sropt.mpi_type = xstrdup(slurm_conf.mpi_default);
 	if (mpi_hook_client_init(sropt.mpi_type) == SLURM_ERROR) {
 		error("invalid MPI type '%s', --mpi=list for acceptable types",
 		      sropt.mpi_type);
@@ -1223,7 +1274,9 @@ static bool _opt_verify(void)
 		opt.x11_magic_cookie = x11_get_xauth();
 	}
 
-	if (opt.gpus_per_socket && (opt.sockets_per_node == NO_VAL)) {
+	/* Validate allocation request only. */
+	if ((sropt.jobid == NO_VAL) &&
+	    opt.gpus_per_socket && (opt.sockets_per_node == NO_VAL)) {
 		error("--gpus-per-socket option requires --sockets-per-node specification");
 		exit(error_exit);
 	}
@@ -1361,7 +1414,7 @@ static void _usage(void)
  	printf(
 "Usage: srun [-N nnodes] [-n ntasks] [-i in] [-o out] [-e err]\n"
 "            [-c ncpus] [-r n] [-p partition] [--hold] [-t minutes]\n"
-"            [-D path] [--immediate[=secs]] [--overcommit] [--no-kill]\n"
+"            [-D path] [--immediate[=secs]] [--overcommit] [--overlap] [--no-kill]\n"
 "            [--oversubscribe] [--label] [--unbuffered] [-m dist] [-J jobname]\n"
 "            [--jobid=id] [--verbose] [--slurmd_debug=#] [--gres=list]\n"
 "            [-T threads] [-W sec] [--gres-flags=opts]\n"
@@ -1373,12 +1426,12 @@ static void _usage(void)
 "            [--cpu-bind=...] [--mem-bind=...] [--network=type]\n"
 "            [--ntasks-per-node=n] [--ntasks-per-socket=n] [reservation=name]\n"
 "            [--ntasks-per-core=n] [--mem-per-cpu=MB] [--preserve-env]\n"
-"            [--profile=...]\n"
+"            [--profile=...] [--exact]\n"
 "            [--mail-type=type] [--mail-user=user] [--nice[=value]]\n"
 "            [--prolog=fname] [--epilog=fname]\n"
 "            [--task-prolog=fname] [--task-epilog=fname]\n"
 "            [--ctrl-comm-ifhn=addr] [--multi-prog] [--mcs-label=mcs]\n"
-"            [--cpu-freq=min[-max[:gov]] [--power=flags] [--spread-job]\n"
+"            [--cpu-freq=min[-max[:gov]]] [--power=flags] [--spread-job]\n"
 "            [--switches=max-switches{@max-time-to-wait}] [--reboot]\n"
 "            [--core-spec=cores] [--thread-spec=threads]\n"
 "            [--bb=burst_buffer_spec] [--bbf=burst_buffer_file]\n"
@@ -1387,7 +1440,7 @@ static void _usage(void)
 "            [-w hosts...] [-x hosts...] [--use-min-nodes]\n"
 "            [--mpi-combine=yes|no] [--het-group=value]\n"
 "            [--cpus-per-gpu=n] [--gpus=n] [--gpu-bind=...] [--gpu-freq=...]\n"
-"            [--gpus-per-node=n] [--gpus-per-socket=n]  [--gpus-per-task=n]\n"
+"            [--gpus-per-node=n] [--gpus-per-socket=n] [--gpus-per-task=n]\n"
 "            [--mem-per-gpu=MB]\n"
 "            executable [args...]\n");
 
@@ -1395,10 +1448,10 @@ static void _usage(void)
 
 static void _help(void)
 {
-	slurm_ctl_conf_t *conf;
+	slurm_conf_t *conf = slurm_conf_lock();
 
         printf (
-"Usage: srun [OPTIONS...] executable [args...]\n"
+"Usage: srun [OPTIONS(0)... [executable(0) [args(0)...]]] [ : [OPTIONS(N)...]] executable(N) [args(N)...]\n"
 "\n"
 "Parallel run options:\n"
 "  -A, --account=name          charge job to specified account\n"
@@ -1456,6 +1509,7 @@ static void _help(void)
 "  -N, --nodes=N               number of nodes on which to run (N = min[-max])\n"
 "  -o, --output=out            location of stdout redirection\n"
 "  -O, --overcommit            overcommit resources\n"
+"      --overlap               Allow other steps to overlap this step\n"
 "      --het-group=value       hetjob component allocation(s) in which to launch\n"
 "                              application\n"
 "  -p, --partition=partition   partition requested\n"
@@ -1510,12 +1564,15 @@ static void _help(void)
 "  -Z, --no-allocate           don't allocate nodes (must supply -w)\n"
 "\n"
 "Consumable resources related options:\n"
-"      --exclusive[=user]      allocate nodes in exclusive mode when\n"
-"                              cpu consumable resource is enabled\n"
-"                              or don't share CPUs for job steps\n"
+"      --exact                 use only the resources requested for the step\n"
+"                              (by default, all non-gres resources on each node\n"
+"                              in the allocation will be used in the step)\n"
+"      --exclusive[=user]      for job allocation, this allocates nodes in\n"
+"                              in exclusive mode\n"
+"                              for job steps, this is equivalent to --exact\n"
 "      --exclusive[=mcs]       allocate nodes in exclusive mode when\n"
 "                              cpu consumable resource is enabled\n"
-"                              and mcs plugin is enabled\n"
+"                              and mcs plugin is enabled (--exact implied)\n"
 "                              or don't share CPUs for job steps\n"
 "      --mem-per-cpu=MB        maximum amount of real memory per allocated\n"
 "                              cpu required by the job.\n"
@@ -1523,16 +1580,19 @@ static void _help(void)
 "      --resv-ports            reserve communication ports\n"
 "\n"
 "Affinity/Multi-core options: (when the task/affinity plugin is enabled)\n"
-"  -B, --extra-node-info=S[:C[:T]]           Expands to:\n"
+"                              For the following 4 options, you are\n"
+"                              specifying the minimum resources available for\n"
+"                              the node(s) allocated to the job.\n"
 "      --sockets-per-node=S    number of sockets per node to allocate\n"
 "      --cores-per-socket=C    number of cores per socket to allocate\n"
 "      --threads-per-core=T    number of threads per core to allocate\n"
-"                              each field can be 'min' or wildcard '*'\n"
-"                              total cpus requested = (N x S x C x T)\n"
+"  -B  --extra-node-info=S[:C[:T]]  combine request of sockets per node,\n"
+"                              cores per socket and threads per core.\n"
+"                              Specify an asterisk (*) as a placeholder,\n"
+"                              a minimum value, or a min-max range.\n"
 "\n"
 "      --ntasks-per-core=n     number of tasks to invoke on each core\n"
 "      --ntasks-per-socket=n   number of tasks to invoke on each socket\n");
-	conf = slurm_conf_lock();
 	if (xstrstr(conf->task_plugin, "affinity") ||
 	    xstrstr(conf->task_plugin, "cgroup")) {
 		printf(
