@@ -1,8 +1,11 @@
 /*****************************************************************************\
- *  test24.1.prog.c - link and test algo of the multifactor plugin.
+ *  test24.4.prog.c - link and test algo of Fair Tree multifactor.
  *
- *  Usage: test24.1.prog
+ *  Usage: test24.4.prog
  *****************************************************************************
+ *  Modified by Brigham Young University
+ *      Ryan Cox <ryan_cox@byu.edu> and Levi Morrison <levi_morrison@byu.edu>
+ *
  *  Copyright (C) 2009 Lawrence Livermore National Security.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Danny Auble <da@llnl.gov>
@@ -30,8 +33,8 @@
 #include <inttypes.h>
 #include <strings.h>
 #include <sys/types.h>
-#include <time.h>
 #include <unistd.h>
+#include <time.h>
 
 #include <slurm/slurm.h>
 #include <slurm/slurm_errno.h>
@@ -49,7 +52,7 @@ int exit_code = 0;
 sshare_time_format_t time_format = SSHARE_TIME_MINS;
 char *time_format_string = "Minutes";
 time_t last_job_update = (time_t) 0;
-uint16_t running_cache = 0;
+uint16_t running_cache = RUNNING_CACHE_STATE_NOTRUNNING;
 
 List   job_list = NULL;		/* job_record list */
 static pthread_mutex_t state_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -81,7 +84,7 @@ int _setup_assoc_list(void)
 	 * we are running off cache */
 	memset(&assoc_init_arg, 0, sizeof(assoc_init_args_t));
 	assoc_init_arg.running_cache = &running_cache;
-	running_cache = 1;
+	running_cache = RUNNING_CACHE_STATE_RUNNING;
 	assoc_mgr_init(NULL, &assoc_init_arg, SLURM_SUCCESS);
 
 	/* Here we make the tres we want to add to the system.
@@ -101,180 +104,374 @@ int _setup_assoc_list(void)
 
 	/* Here we make the associations we want to add to the system.
 	 * We do this as an update to avoid having to do setup. */
+	memset(&update, 0, sizeof(slurmdb_update_object_t));
 	update.type = SLURMDB_ADD_ASSOC;
 	update.objects = list_create(slurmdb_destroy_assoc_rec);
 
-	/* Just so we don't have to worry about lft's and rgt's we
-	 * will just append these on in order.
-	 * Note: the commented out lfts and rgts as of 10-29-10 are
-	 * correct.  By doing an append they go on
-	 * sorted in hierarchy order.  The sort that happens inside
-	 * the internal slurm code will sort alpha automatically, (You can
-	 * test this by putting AccountF before AccountE.
-	 */
-
-	/* First only add the accounts */
-	/* root association */
+	/* root assoc */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 1;
-	/* assoc->lft = 1; */
-	/* assoc->rgt = 28; */
 	assoc->acct = xstrdup("root");
 	list_append(update.objects, assoc);
 
-	/* sub of root id 1 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 2;
 	assoc->parent_id = 1;
 	assoc->shares_raw = 40;
-	/* assoc->lft = 2; */
-	/* assoc->rgt = 13; */
-	assoc->acct = xstrdup("AccountA");
+	assoc->acct = xstrdup("aA");
 	list_append(update.objects, assoc);
 
-	/* sub of AccountA id 2 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 21;
-	/* assoc->lft = 3; */
-	/* assoc->rgt = 6; */
 	assoc->parent_id = 2;
 	assoc->shares_raw = 30;
-	assoc->acct = xstrdup("AccountB");
+	assoc->acct = xstrdup("aAA");
 	list_append(update.objects, assoc);
 
-	/* sub of AccountB id 21 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 211;
-	/* assoc->lft = 4; */
-	/* assoc->rgt = 5; */
 	assoc->parent_id = 21;
 	assoc->shares_raw = 1;
 	assoc->usage->usage_raw = 20;
-	assoc->acct = xstrdup("AccountB");
-	assoc->user = xstrdup("User1");
+	assoc->acct = xstrdup("aAA");
+	assoc->user = xstrdup("uAA1");
 	list_append(update.objects, assoc);
 
-	/* sub of AccountA id 2 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 22;
-	/* assoc->lft = 7; */
-	/* assoc->rgt = 12; */
 	assoc->parent_id = 2;
 	assoc->shares_raw = 10;
-	assoc->acct = xstrdup("AccountC");
+	assoc->acct = xstrdup("aAB");
 	list_append(update.objects, assoc);
 
-	/* sub of AccountC id 22 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 221;
-	/* assoc->lft = 8; */
-	/* assoc->rgt = 9; */
 	assoc->parent_id = 22;
 	assoc->shares_raw = 1;
 	assoc->usage->usage_raw = 25;
-	assoc->acct = xstrdup("AccountC");
-	assoc->user = xstrdup("User2");
+	assoc->acct = xstrdup("aAB");
+	assoc->user = xstrdup("uAB1");
 	list_append(update.objects, assoc);
 
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 222;
-	/* assoc->lft = 10; */
-	/* assoc->rgt = 11; */
 	assoc->parent_id = 22;
 	assoc->shares_raw = 1;
 	assoc->usage->usage_raw = 0;
-	assoc->acct = xstrdup("AccountC");
-	assoc->user = xstrdup("User3");
+	assoc->acct = xstrdup("aAB");
+	assoc->user = xstrdup("uAB2");
 	list_append(update.objects, assoc);
 
-	/* sub of root id 1 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 3;
-	/* assoc->lft = 14; */
-	/* assoc->rgt = 23; */
 	assoc->parent_id = 1;
 	assoc->shares_raw = 60;
-	assoc->acct = xstrdup("AccountD");
+	assoc->acct = xstrdup("aB");
 	list_append(update.objects, assoc);
 
-	/* sub of AccountD id 3 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 31;
-	/* assoc->lft = 19; */
-	/* assoc->rgt = 22; */
 	assoc->parent_id = 3;
 	assoc->shares_raw = 25;
-	assoc->acct = xstrdup("AccountE");
+	assoc->acct = xstrdup("aBA");
 	list_append(update.objects, assoc);
 
-	/* sub of AccountE id 31 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 311;
-	/* assoc->lft = 20; */
-	/* assoc->rgt = 21; */
 	assoc->parent_id = 31;
 	assoc->shares_raw = 1;
 	assoc->usage->usage_raw = 25;
-	assoc->acct = xstrdup("AccountE");
-	assoc->user = xstrdup("User4");
+	assoc->acct = xstrdup("aBA");
+	assoc->user = xstrdup("uBA1");
 	list_append(update.objects, assoc);
 
-	/* sub of AccountD id 3 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 32;
-	/* assoc->lft = 15; */
-	/* assoc->rgt = 18; */
 	assoc->parent_id = 3;
 	assoc->shares_raw = 35;
-	assoc->acct = xstrdup("AccountF");
+	assoc->acct = xstrdup("aBB");
 	list_append(update.objects, assoc);
 
-	/* sub of AccountF id 32 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 321;
-	/* assoc->lft = 16; */
-	/* assoc->rgt = 17; */
 	assoc->parent_id = 32;
 	assoc->shares_raw = 1;
 	assoc->usage->usage_raw = 0;
-	assoc->acct = xstrdup("AccountF");
-	assoc->user = xstrdup("User5");
+	assoc->acct = xstrdup("aBB");
+	assoc->user = xstrdup("uBB1");
 	list_append(update.objects, assoc);
 
-	/* sub of root id 1 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 4;
-	/* assoc->lft = 24; */
-	/* assoc->rgt = 27; */
 	assoc->parent_id = 1;
 	assoc->shares_raw = 0;
-	assoc->acct = xstrdup("AccountG");
+	assoc->usage->usage_raw = 30;
+	assoc->acct = xstrdup("aC");
 	list_append(update.objects, assoc);
 
-	/* sub of AccountG id 4 */
 	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
 	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
 	assoc->id = 41;
-	/* assoc->lft = 25; */
-	/* assoc->rgt = 26; */
 	assoc->parent_id = 4;
 	assoc->shares_raw = 0;
 	assoc->usage->usage_raw = 30;
-	assoc->acct = xstrdup("AccountG");
-	assoc->user = xstrdup("User6");
+	assoc->acct = xstrdup("aC");
+	assoc->user = xstrdup("uC1");
+	list_append(update.objects, assoc);
+
+	/* Check for proper handling of Fairshare=parent */
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 5;
+	assoc->parent_id = 1;
+	assoc->shares_raw = 50;
+	assoc->acct = xstrdup("aD");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 51;
+	assoc->parent_id = 5;
+	assoc->shares_raw = SLURMDB_FS_USE_PARENT;
+	assoc->usage->usage_raw = 35;
+	assoc->acct = xstrdup("aDA");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 511;
+	assoc->parent_id = 51;
+	assoc->shares_raw = SLURMDB_FS_USE_PARENT;
+	assoc->usage->usage_raw = 10;
+	assoc->acct = xstrdup("aDA");
+	assoc->user = xstrdup("uDA1");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 512;
+	assoc->parent_id = 51;
+	assoc->shares_raw = 30;
+	assoc->usage->usage_raw = 10;
+	assoc->acct = xstrdup("aDA");
+	assoc->user = xstrdup("uDA2");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 513;
+	assoc->parent_id = 51;
+	assoc->shares_raw = 50;
+	assoc->usage->usage_raw = 25;
+	assoc->acct = xstrdup("aDA");
+	assoc->user = xstrdup("uDA3");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 52;
+	assoc->parent_id = 5;
+	assoc->shares_raw = SLURMDB_FS_USE_PARENT;
+	assoc->usage->usage_raw = 20;
+	assoc->acct = xstrdup("aD");
+	assoc->user = xstrdup("uD1");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 53;
+	assoc->parent_id = 5;
+	assoc->shares_raw = 40;
+	assoc->usage->usage_raw = 20;
+	assoc->acct = xstrdup("aD");
+	assoc->user = xstrdup("uD2");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 54;
+	assoc->parent_id = 5;
+	assoc->shares_raw = 50;
+	assoc->usage->usage_raw = 25;
+	assoc->acct = xstrdup("aD");
+	assoc->user = xstrdup("uD3");
+	list_append(update.objects, assoc);
+
+	/* Check for proper tie handling */
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 6;
+	assoc->parent_id = 1;
+	assoc->shares_raw = 10;
+	assoc->usage->usage_raw = 0;
+	assoc->acct = xstrdup("aE");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 61;
+	assoc->parent_id = 6;
+	assoc->shares_raw = 10;
+	assoc->usage->usage_raw = 0;
+	assoc->acct = xstrdup("aE");
+	assoc->user = xstrdup("aE1");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 62;
+	assoc->parent_id = 6;
+	assoc->shares_raw = 10;
+	assoc->usage->usage_raw = 0;
+	assoc->acct = xstrdup("aE");
+	assoc->user = xstrdup("aE2");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 7;
+	assoc->parent_id = 1;
+	assoc->shares_raw = 10;
+	assoc->usage->usage_raw = 0;
+	assoc->acct = xstrdup("root");
+	assoc->user = xstrdup("u1");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 8;
+	assoc->parent_id = 1;
+	assoc->shares_raw = 20;
+	assoc->usage->usage_raw = 0;
+	assoc->acct = xstrdup("aF");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 81;
+	assoc->parent_id = 8;
+	assoc->shares_raw = 10;
+	assoc->usage->usage_raw = 0;
+	assoc->acct = xstrdup("aF");
+	assoc->user = xstrdup("uF1");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 82;
+	assoc->parent_id = 8;
+	assoc->shares_raw = 20;
+	assoc->usage->usage_raw = 0;
+	assoc->acct = xstrdup("aF");
+	assoc->user = xstrdup("uF2");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 9;
+	assoc->parent_id = 1;
+	assoc->shares_raw = 8;
+	assoc->usage->usage_raw = 20;
+	assoc->acct = xstrdup("aG");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 91;
+	assoc->parent_id = 9;
+	assoc->shares_raw = 10;
+	assoc->usage->usage_raw = 10;
+	assoc->acct = xstrdup("aG");
+	assoc->user = xstrdup("uG1");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 92;
+	assoc->parent_id = 9;
+	assoc->shares_raw = 10;
+	assoc->usage->usage_raw = 10;
+	assoc->acct = xstrdup("aGA");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 921;
+	assoc->parent_id = 92;
+	assoc->shares_raw = 20;
+	assoc->usage->usage_raw = 4;
+	assoc->acct = xstrdup("aGA");
+	assoc->user = xstrdup("uGA1");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 921;
+	assoc->parent_id = 92;
+	assoc->shares_raw = 20;
+	assoc->usage->usage_raw = 6;
+	assoc->acct = xstrdup("aGA");
+	assoc->user = xstrdup("uGA2");
+	list_append(update.objects, assoc);
+
+	assoc = xmalloc(sizeof(slurmdb_assoc_rec_t));
+	assoc->cluster = xstrdup(slurm_conf.cluster_name);
+	assoc->usage = slurmdb_create_assoc_usage(g_tres_count);
+	assoc->id = 1001;
+	assoc->parent_id = 1;
+	assoc->shares_raw = 10;
+	assoc->usage->usage_raw = 10;
+	assoc->acct = xstrdup("root");
+	assoc->user = xstrdup("u2");
 	list_append(update.objects, assoc);
 
 	if (assoc_mgr_update_assocs(&update, false))
@@ -287,11 +484,11 @@ int _setup_assoc_list(void)
 int main (int argc, char **argv)
 {
 	log_options_t logopt = LOG_OPTS_STDERR_ONLY;
-	slurm_ctl_conf_t *conf = NULL;
+	slurm_conf_t *conf = NULL;
 	shares_response_msg_t resp;
 
 	log_init(xbasename(argv[0]), logopt, 0, NULL);
-	xfree(slurmctld_conf.priority_type);
+	xfree(slurm_conf.priority_type);
 	//logopt.stderr_level += 5;
 	logopt.prefix_level = 1;
 	log_alter(logopt, 0, NULL);
@@ -302,7 +499,7 @@ int main (int argc, char **argv)
 	/* force priority type to be multifactor */
 	xfree(conf->priority_type);
 	conf->priority_type = xstrdup("priority/multifactor");
-	conf->priority_flags = 0;
+	conf->priority_flags = PRIORITY_FLAGS_FAIR_TREE;
 	/* force accounting type to be slurmdbd (It doesn't really talk
 	 * to any database, but needs this to work with fairshare
 	 * calculation). */
@@ -320,13 +517,12 @@ int main (int argc, char **argv)
 	conf->priority_weight_js = 0;
 	conf->priority_weight_part = 0;
 	conf->priority_weight_qos = 0;
-	xfree(conf->priority_weight_tres);
 	slurm_conf_unlock();
 
 	/* we don't want to do any decay here so make the save state
 	 * to /dev/null */
-	xfree(slurmctld_conf.state_save_location);
-	slurmctld_conf.state_save_location = "/dev/null";
+	xfree(slurm_conf.state_save_location);
+	slurm_conf.state_save_location = "/dev/null";
 	/* now set up the association tree */
 	_setup_assoc_list();
 	/* now set up the job list */
@@ -359,5 +555,6 @@ int main (int argc, char **argv)
 		list_destroy(assoc_mgr_assoc_list);
 	if (assoc_mgr_qos_list)
 		list_destroy(assoc_mgr_qos_list);
+	xfree(assoc_mgr_tres_array);
 	return 0;
 }
